@@ -34,19 +34,19 @@ try:
     # Python 2.7
     from urllib import unquote as _unquote
     from urllib import quote as _quote
-    from urllib import urlencode as _urlencode
 
 except ImportError:
     # Python 3.x
     from urllib.parse import unquote as _unquote
     from urllib.parse import quote as _quote
-    from urllib.parse import urlencode as _urlencode
 
 from .AppriseLocale import gettext_lazy as _
 from .AppriseAsset import AppriseAsset
+from .utils import urlencode
 from .utils import parse_url
 from .utils import parse_bool
 from .utils import parse_list
+from .utils import parse_phone_no
 
 # Used to break a path list into parts
 PATHSPLIT_LIST_DELIM = re.compile(r'[ \t\r\n,\\/]+')
@@ -358,7 +358,7 @@ class URLBase(object):
                 .replace(u' ', u'&nbsp;')
 
         if convert_new_lines:
-            return escaped.replace(u'\n', u'&lt;br/&gt;')
+            return escaped.replace(u'\n', u'<br/>')
 
         return escaped
 
@@ -496,17 +496,8 @@ class URLBase(object):
         Returns:
             str: The escaped parameters returned as a string
         """
-        # Tidy query by eliminating any records set to None
-        _query = {k: v for (k, v) in query.items() if v is not None}
-        try:
-            # Python v3.x
-            return _urlencode(
-                _query, doseq=doseq, safe=safe, encoding=encoding,
-                errors=errors)
-
-        except TypeError:
-            # Python v2.7
-            return _urlencode(_query)
+        return urlencode(
+            query, doseq=doseq, safe=safe, encoding=encoding, errors=errors)
 
     @staticmethod
     def split_path(path, unquote=True):
@@ -560,6 +551,39 @@ class URLBase(object):
 
         return content
 
+    @staticmethod
+    def parse_phone_no(content, unquote=True):
+        """A wrapper to utils.parse_phone_no() with unquoting support
+
+        Parses a specified set of data and breaks it into a list.
+
+        Args:
+            content (str): The path to split up into a list. If a list is
+                 provided, then it's individual entries are processed.
+
+            unquote (:obj:`bool`, optional): call unquote on each element
+                 added to the returned list.
+
+        Returns:
+            list: A unique list containing all of the elements in the path
+        """
+
+        if unquote:
+            try:
+                content = URLBase.unquote(content)
+            except TypeError:
+                # Nothing further to do
+                return []
+
+            except AttributeError:
+                # This exception ONLY gets thrown under Python v2.7 if an
+                # object() is passed in place of the content
+                return []
+
+        content = parse_phone_no(content)
+
+        return content
+
     @property
     def app_id(self):
         return self.asset.app_id if self.asset.app_id else ''
@@ -636,6 +660,8 @@ class URLBase(object):
                 results['qsd'].get('verify', True))
 
         # Password overrides
+        if 'password' in results['qsd']:
+            results['password'] = results['qsd']['password']
         if 'pass' in results['qsd']:
             results['password'] = results['qsd']['pass']
 
@@ -651,6 +677,9 @@ class URLBase(object):
         if 'cto' in results['qsd']:
             results['socket_connect_timeout'] = results['qsd']['cto']
 
+        if 'port' in results['qsd']:
+            results['port'] = results['qsd']['port']
+
         return results
 
     @staticmethod
