@@ -30,16 +30,14 @@ import yaml
 import time
 
 from .. import plugins
+from .. import common
 from ..AppriseAsset import AppriseAsset
 from ..URLBase import URLBase
-from ..common import ConfigFormat
-from ..common import CONFIG_FORMATS
-from ..common import ContentIncludeMode
 from ..utils import GET_SCHEMA_RE
 from ..utils import parse_list
 from ..utils import parse_bool
 from ..utils import parse_urls
-from . import SCHEMA_MAP
+from ..utils import cwe312_url
 
 # Test whether token is valid or not
 VALID_TOKEN = re.compile(
@@ -56,7 +54,7 @@ class ConfigBase(URLBase):
 
     # The default expected configuration format unless otherwise
     # detected by the sub-modules
-    default_config_format = ConfigFormat.TEXT
+    default_config_format = common.ConfigFormat.TEXT
 
     # This is only set if the user overrides the config format on the URL
     # this should always initialize itself as None
@@ -69,7 +67,7 @@ class ConfigBase(URLBase):
 
     # By default all configuration is not includable using the 'include'
     # line found in configuration files.
-    allow_cross_includes = ContentIncludeMode.NEVER
+    allow_cross_includes = common.ContentIncludeMode.NEVER
 
     # the config path manages the handling of relative include
     config_path = os.getcwd()
@@ -141,7 +139,7 @@ class ConfigBase(URLBase):
             # Store the enforced config format
             self.config_format = kwargs.get('format').lower()
 
-            if self.config_format not in CONFIG_FORMATS:
+            if self.config_format not in common.CONFIG_FORMATS:
                 # Simple error checking
                 err = 'An invalid config format ({}) was specified.'.format(
                     self.config_format)
@@ -209,8 +207,8 @@ class ConfigBase(URLBase):
         # Configuration files were detected; recursively populate them
         # If we have been configured to do so
         for url in configs:
-            if self.recursion > 0:
 
+            if self.recursion > 0:
                 # Attempt to acquire the schema at the very least to allow
                 # our configuration based urls.
                 schema = GET_SCHEMA_RE.match(url)
@@ -223,37 +221,43 @@ class ConfigBase(URLBase):
                         url = os.path.join(self.config_path, url)
 
                     url = '{}://{}'.format(schema, URLBase.quote(url))
+
                 else:
                     # Ensure our schema is always in lower case
                     schema = schema.group('schema').lower()
 
                     # Some basic validation
-                    if schema not in SCHEMA_MAP:
+                    if schema not in common.CONFIG_SCHEMA_MAP:
                         ConfigBase.logger.warning(
                             'Unsupported include schema {}.'.format(schema))
                         continue
 
+                # CWE-312 (Secure Logging) Handling
+                loggable_url = url if not asset.secure_logging \
+                    else cwe312_url(url)
+
                 # Parse our url details of the server object as dictionary
                 # containing all of the information parsed from our URL
-                results = SCHEMA_MAP[schema].parse_url(url)
+                results = common.CONFIG_SCHEMA_MAP[schema].parse_url(url)
                 if not results:
                     # Failed to parse the server URL
                     self.logger.warning(
-                        'Unparseable include URL {}'.format(url))
+                        'Unparseable include URL {}'.format(loggable_url))
                     continue
 
                 # Handle cross inclusion based on allow_cross_includes rules
-                if (SCHEMA_MAP[schema].allow_cross_includes ==
-                        ContentIncludeMode.STRICT
+                if (common.CONFIG_SCHEMA_MAP[schema].allow_cross_includes ==
+                        common.ContentIncludeMode.STRICT
                         and schema not in self.schemas()
                         and not self.insecure_includes) or \
-                        SCHEMA_MAP[schema].allow_cross_includes == \
-                        ContentIncludeMode.NEVER:
+                        common.CONFIG_SCHEMA_MAP[schema] \
+                        .allow_cross_includes == \
+                        common.ContentIncludeMode.NEVER:
 
                     # Prevent the loading if insecure base protocols
                     ConfigBase.logger.warning(
                         'Including {}:// based configuration is prohibited. '
-                        'Ignoring URL {}'.format(schema, url))
+                        'Ignoring URL {}'.format(schema, loggable_url))
                     continue
 
                 # Prepare our Asset Object
@@ -274,12 +278,13 @@ class ConfigBase(URLBase):
                 try:
                     # Attempt to create an instance of our plugin using the
                     # parsed URL information
-                    cfg_plugin = SCHEMA_MAP[results['schema']](**results)
+                    cfg_plugin = \
+                        common.CONFIG_SCHEMA_MAP[results['schema']](**results)
 
                 except Exception as e:
                     # the arguments are invalid or can not be used.
                     self.logger.warning(
-                        'Could not load include URL: {}'.format(url))
+                        'Could not load include URL: {}'.format(loggable_url))
                     self.logger.debug('Loading Exception: {}'.format(str(e)))
                     continue
 
@@ -292,16 +297,23 @@ class ConfigBase(URLBase):
                 del cfg_plugin
 
             else:
+                # CWE-312 (Secure Logging) Handling
+                loggable_url = url if not asset.secure_logging \
+                    else cwe312_url(url)
+
                 self.logger.debug(
-                    'Recursion limit reached; ignoring Include URL: %s' % url)
+                    'Recursion limit reached; ignoring Include URL: %s',
+                    loggable_url)
 
         if self._cached_servers:
-            self.logger.info('Loaded {} entries from {}'.format(
-                len(self._cached_servers), self.url()))
+            self.logger.info(
+                'Loaded {} entries from {}'.format(
+                    len(self._cached_servers),
+                    self.url(privacy=asset.secure_logging)))
         else:
             self.logger.warning(
                 'Failed to load Apprise configuration from {}'.format(
-                    self.url()))
+                    self.url(privacy=asset.secure_logging)))
 
         # Set the time our content was cached at
         self._cached_time = time.time()
@@ -366,7 +378,7 @@ class ConfigBase(URLBase):
         # Allow overriding the default config format
         if 'format' in results['qsd']:
             results['format'] = results['qsd'].get('format')
-            if results['format'] not in CONFIG_FORMATS:
+            if results['format'] not in common.CONFIG_FORMATS:
                 URLBase.logger.warning(
                     'Unsupported format specified {}'.format(
                         results['format']))
@@ -444,14 +456,14 @@ class ConfigBase(URLBase):
 
             # Attempt to detect configuration
             if result.group('yaml'):
-                config_format = ConfigFormat.YAML
+                config_format = common.ConfigFormat.YAML
                 ConfigBase.logger.debug(
                     'Detected YAML configuration '
                     'based on line {}.'.format(line))
                 break
 
             elif result.group('text'):
-                config_format = ConfigFormat.TEXT
+                config_format = common.ConfigFormat.TEXT
                 ConfigBase.logger.debug(
                     'Detected TEXT configuration '
                     'based on line {}.'.format(line))
@@ -459,7 +471,7 @@ class ConfigBase(URLBase):
 
             # If we reach here, we have a comment entry
             # Adjust default format to TEXT
-            config_format = ConfigFormat.TEXT
+            config_format = common.ConfigFormat.TEXT
 
         return config_format
 
@@ -480,7 +492,7 @@ class ConfigBase(URLBase):
                 ConfigBase.logger.error('Could not detect configuration')
                 return (list(), list())
 
-        if config_format not in CONFIG_FORMATS:
+        if config_format not in common.CONFIG_FORMATS:
             # Invalid configuration type specified
             ConfigBase.logger.error(
                 'An invalid configuration format ({}) was specified'.format(
@@ -531,6 +543,9 @@ class ConfigBase(URLBase):
         # the include keyword
         configs = list()
 
+        # Prepare our Asset Object
+        asset = asset if isinstance(asset, AppriseAsset) else AppriseAsset()
+
         # Define what a valid line should look like
         valid_line_re = re.compile(
             r'^\s*(?P<line>([;#]+(?P<comment>.*))|'
@@ -567,41 +582,52 @@ class ConfigBase(URLBase):
                 continue
 
             if config:
-                ConfigBase.logger.debug('Include URL: {}'.format(config))
+                # CWE-312 (Secure Logging) Handling
+                loggable_url = config if not asset.secure_logging \
+                    else cwe312_url(config)
+
+                ConfigBase.logger.debug(
+                    'Include URL: {}'.format(loggable_url))
 
                 # Store our include line
                 configs.append(config.strip())
                 continue
 
+            # CWE-312 (Secure Logging) Handling
+            loggable_url = url if not asset.secure_logging \
+                else cwe312_url(url)
+
             # Acquire our url tokens
-            results = plugins.url_to_dict(url)
+            results = plugins.url_to_dict(
+                url, secure_logging=asset.secure_logging)
             if results is None:
                 # Failed to parse the server URL
                 ConfigBase.logger.warning(
-                    'Unparseable URL {} on line {}.'.format(url, line))
+                    'Unparseable URL {} on line {}.'.format(
+                        loggable_url, line))
                 continue
 
             # Build a list of tags to associate with the newly added
             # notifications if any were set
             results['tag'] = set(parse_list(result.group('tags')))
 
-            # Prepare our Asset Object
-            results['asset'] = \
-                asset if isinstance(asset, AppriseAsset) else AppriseAsset()
+            # Set our Asset Object
+            results['asset'] = asset
 
             try:
                 # Attempt to create an instance of our plugin using the
                 # parsed URL information
-                plugin = plugins.SCHEMA_MAP[results['schema']](**results)
+                plugin = common.NOTIFY_SCHEMA_MAP[results['schema']](**results)
 
                 # Create log entry of loaded URL
-                ConfigBase.logger.debug('Loaded URL: {}'.format(plugin.url()))
+                ConfigBase.logger.debug(
+                    'Loaded URL: %s', plugin.url(privacy=asset.secure_logging))
 
             except Exception as e:
                 # the arguments are invalid or can not be used.
                 ConfigBase.logger.warning(
                     'Could not load URL {} on line {}.'.format(
-                        url, line))
+                        loggable_url, line))
                 ConfigBase.logger.debug('Loading Exception: %s' % str(e))
                 continue
 
@@ -756,6 +782,10 @@ class ConfigBase(URLBase):
             # we can. Reset it to None on each iteration
             results = list()
 
+            # CWE-312 (Secure Logging) Handling
+            loggable_url = url if not asset.secure_logging \
+                else cwe312_url(url)
+
             if isinstance(url, six.string_types):
                 # We're just a simple URL string...
                 schema = GET_SCHEMA_RE.match(url)
@@ -764,16 +794,18 @@ class ConfigBase(URLBase):
                     # config file at least has something to take action
                     # with.
                     ConfigBase.logger.warning(
-                        'Invalid URL {}, entry #{}'.format(url, no + 1))
+                        'Invalid URL {}, entry #{}'.format(
+                            loggable_url, no + 1))
                     continue
 
                 # We found a valid schema worthy of tracking; store it's
                 # details:
-                _results = plugins.url_to_dict(url)
+                _results = plugins.url_to_dict(
+                    url, secure_logging=asset.secure_logging)
                 if _results is None:
                     ConfigBase.logger.warning(
                         'Unparseable URL {}, entry #{}'.format(
-                            url, no + 1))
+                            loggable_url, no + 1))
                     continue
 
                 # add our results to our global set
@@ -819,7 +851,8 @@ class ConfigBase(URLBase):
                         'Unsupported URL, entry #{}'.format(no + 1))
                     continue
 
-                _results = plugins.url_to_dict(_url)
+                _results = plugins.url_to_dict(
+                    _url, secure_logging=asset.secure_logging)
                 if _results is None:
                     # Setup dictionary
                     _results = {
@@ -847,8 +880,8 @@ class ConfigBase(URLBase):
                                 del entries['schema']
 
                             # support our special tokens (if they're present)
-                            if schema in plugins.SCHEMA_MAP:
-                                entries = ConfigBase.__extract_special_tokens(
+                            if schema in common.NOTIFY_SCHEMA_MAP:
+                                entries = ConfigBase._special_token_handler(
                                     schema, entries)
 
                             # Extend our dictionary with our new entries
@@ -859,8 +892,8 @@ class ConfigBase(URLBase):
 
                 elif isinstance(tokens, dict):
                     # support our special tokens (if they're present)
-                    if schema in plugins.SCHEMA_MAP:
-                        tokens = ConfigBase.__extract_special_tokens(
+                    if schema in common.NOTIFY_SCHEMA_MAP:
+                        tokens = ConfigBase._special_token_handler(
                             schema, tokens)
 
                     # Copy ourselves a template of our parsed URL as a base to
@@ -893,6 +926,14 @@ class ConfigBase(URLBase):
                 # Grab our first item
                 _results = results.pop(0)
 
+                if _results['schema'] not in common.NOTIFY_SCHEMA_MAP:
+                    # the arguments are invalid or can not be used.
+                    ConfigBase.logger.warning(
+                        'An invalid Apprise schema ({}) in YAML configuration '
+                        'entry #{}, item #{}'
+                        .format(_results['schema'], no + 1, entry))
+                    continue
+
                 # tag is a special keyword that is managed by Apprise object.
                 # The below ensures our tags are set correctly
                 if 'tag' in _results:
@@ -924,14 +965,17 @@ class ConfigBase(URLBase):
                 # Prepare our Asset Object
                 _results['asset'] = asset
 
+                # Now we generate our plugin
                 try:
                     # Attempt to create an instance of our plugin using the
                     # parsed URL information
-                    plugin = plugins.SCHEMA_MAP[_results['schema']](**_results)
+                    plugin = common.\
+                        NOTIFY_SCHEMA_MAP[_results['schema']](**_results)
 
                     # Create log entry of loaded URL
                     ConfigBase.logger.debug(
-                        'Loaded URL: {}'.format(plugin.url()))
+                        'Loaded URL: {}'.format(
+                            plugin.url(privacy=asset.secure_logging)))
 
                 except Exception as e:
                     # the arguments are invalid or can not be used.
@@ -962,7 +1006,7 @@ class ConfigBase(URLBase):
         return self._cached_servers.pop(index)
 
     @staticmethod
-    def __extract_special_tokens(schema, tokens):
+    def _special_token_handler(schema, tokens):
         """
         This function takes a list of tokens and updates them to no longer
         include any special tokens such as +,-, and :
@@ -979,7 +1023,7 @@ class ConfigBase(URLBase):
         # Create a copy of our dictionary
         tokens = tokens.copy()
 
-        for kw, meta in plugins.SCHEMA_MAP[schema]\
+        for kw, meta in common.NOTIFY_SCHEMA_MAP[schema]\
                 .template_kwargs.items():
 
             # Determine our prefix:
@@ -994,7 +1038,7 @@ class ConfigBase(URLBase):
                 # we're done with this entry
                 continue
 
-            if not isinstance(tokens.get(kw, None), dict):
+            if not isinstance(tokens.get(kw), dict):
                 # Invalid; correct it
                 tokens[kw] = dict()
 
@@ -1005,6 +1049,88 @@ class ConfigBase(URLBase):
             # Update our entries
             tokens[kw].update(matches)
 
+        # Now map our tokens accordingly to the class templates defined by
+        # each service.
+        #
+        # This is specifically used for YAML file parsing.  It allows a user to
+        # define an entry such as:
+        #
+        # urls:
+        #   - mailto://user:pass@domain:
+        #       - to: user1@hotmail.com
+        #       - to: user2@hotmail.com
+        #
+        # Under the hood, the NotifyEmail() class does not parse the `to`
+        # argument. It's contents needs to be mapped to `targets`.  This is
+        # defined in the class via the `template_args` and template_tokens`
+        # section.
+        #
+        # This function here allows these mappings to take place within the
+        # YAML file as independant arguments.
+        class_templates = \
+            plugins.details(common.NOTIFY_SCHEMA_MAP[schema])
+
+        for key in list(tokens.keys()):
+
+            if key not in class_templates['args']:
+                # No need to handle non-arg entries
+                continue
+
+            # get our `map_to` and/or 'alias_of' value (if it exists)
+            map_to = class_templates['args'][key].get(
+                'alias_of', class_templates['args'][key].get('map_to', ''))
+
+            if map_to == key:
+                # We're already good as we are now
+                continue
+
+            if map_to in class_templates['tokens']:
+                meta = class_templates['tokens'][map_to]
+
+            else:
+                meta = class_templates['args'].get(
+                    map_to, class_templates['args'][key])
+
+            # Perform a translation/mapping if our code reaches here
+            value = tokens[key]
+            del tokens[key]
+
+            # Detect if we're dealign with a list or not
+            is_list = re.search(
+                r'^list:.*',
+                meta.get('type'),
+                re.IGNORECASE)
+
+            if map_to not in tokens:
+                tokens[map_to] = [] if is_list \
+                    else meta.get('default')
+
+            elif is_list and not isinstance(tokens.get(map_to), list):
+                # Convert ourselves to a list if we aren't already
+                tokens[map_to] = [tokens[map_to]]
+
+            # Type Conversion
+            if re.search(
+                    r'^(choice:)?string',
+                    meta.get('type'),
+                    re.IGNORECASE) \
+                    and not isinstance(value, six.string_types):
+
+                # Ensure our format is as expected
+                value = str(value)
+
+            # Apply any further translations if required (absolute map)
+            # This is the case when an arg maps to a token which further
+            # maps to a different function arg on the class constructor
+            abs_map = meta.get('map_to', map_to)
+
+            # Set our token as how it was provided by the configuration
+            if isinstance(tokens.get(map_to), list):
+                tokens[abs_map].append(value)
+
+            else:
+                tokens[abs_map] = value
+
         # Return our tokens
         return tokens
 
