@@ -91,21 +91,6 @@ EMAIL_TEMPLATES = (
         },
     ),
 
-    # Pronto Mail
-    (
-        'Pronto Mail',
-        re.compile(
-            r'^((?P<label>[^+]+)\+)?(?P<id>[^@]+)@'
-            r'(?P<domain>prontomail\.com)$', re.I),
-        {
-            'port': 465,
-            'smtp_host': 'secure.emailsrvr.com',
-            'secure': True,
-            'secure_mode': SecureMailMode.STARTTLS,
-            'login_type': (WebBaseLogin.EMAIL, )
-        },
-    ),
-
     # Yandex
     (
         'Yandex',
@@ -126,10 +111,10 @@ EMAIL_TEMPLATES = (
         'Microsoft Hotmail',
         re.compile(
             r'^((?P<label>[^+]+)\+)?(?P<id>[^@]+)@'
-            r'(?P<domain>(hotmail|live)\.com)$', re.I),
+            r'(?P<domain>(outlook|hotmail|live)\.com(\.au)?)$', re.I),
         {
             'port': 587,
-            'smtp_host': 'smtp.live.com',
+            'smtp_host': 'smtp-mail.outlook.com',
             'secure': True,
             'secure_mode': SecureMailMode.STARTTLS,
             'login_type': (WebBaseLogin.EMAIL, )
@@ -235,7 +220,6 @@ EMAIL_TEMPLATES = (
         },
     ),
 
-
     # SendGrid (Email Server)
     # You must specify an authenticated sender address in the from= settings
     # and a valid email in the to= to deliver your emails to
@@ -253,6 +237,36 @@ EMAIL_TEMPLATES = (
         },
     ),
 
+    # 163.com
+    (
+        '163.com',
+        re.compile(
+            r'^((?P<label>[^+]+)\+)?(?P<id>[^@]+)@'
+            r'(?P<domain>163\.com)$', re.I),
+        {
+            'port': 465,
+            'smtp_host': 'smtp.163.com',
+            'secure': True,
+            'secure_mode': SecureMailMode.SSL,
+            'login_type': (WebBaseLogin.EMAIL, )
+        },
+    ),
+
+    # Foxmail.com
+    (
+        'Foxmail.com',
+        re.compile(
+            r'^((?P<label>[^+]+)\+)?(?P<id>[^@]+)@'
+            r'(?P<domain>(foxmail|qq)\.com)$', re.I),
+        {
+            'port': 587,
+            'smtp_host': 'smtp.qq.com',
+            'secure': True,
+            'secure_mode': SecureMailMode.STARTTLS,
+            'login_type': (WebBaseLogin.EMAIL, )
+        },
+    ),
+
     # Catch All
     (
         'Custom',
@@ -520,7 +534,7 @@ class NotifyEmail(NotifyBase):
             )
 
         # Apply any defaults based on certain known configurations
-        self.NotifyEmailDefaults()
+        self.NotifyEmailDefaults(secure_mode=secure_mode, **kwargs)
 
         # if there is still no smtp_host then we fall back to the hostname
         if not self.smtp_host:
@@ -528,7 +542,7 @@ class NotifyEmail(NotifyBase):
 
         return
 
-    def NotifyEmailDefaults(self):
+    def NotifyEmailDefaults(self, secure_mode=None, port=None, **kwargs):
         """
         A function that prefills defaults based on the email
         it was provided.
@@ -557,18 +571,23 @@ class NotifyEmail(NotifyBase):
                     'Applying %s Defaults' %
                     EMAIL_TEMPLATES[i][0],
                 )
-                self.port = EMAIL_TEMPLATES[i][2]\
-                    .get('port', self.port)
+                # the secure flag can not be altered if defined in the template
                 self.secure = EMAIL_TEMPLATES[i][2]\
                     .get('secure', self.secure)
-                self.secure_mode = EMAIL_TEMPLATES[i][2]\
-                    .get('secure_mode', self.secure_mode)
+
+                # The SMTP Host check is already done above; if it was
+                # specified we wouldn't even reach this part of the code.
                 self.smtp_host = EMAIL_TEMPLATES[i][2]\
                     .get('smtp_host', self.smtp_host)
 
-                if self.smtp_host is None:
-                    # default to our host
-                    self.smtp_host = self.host
+                # The following can be over-ridden if defined manually in the
+                # Apprise URL.  Otherwise they take on the template value
+                if not port:
+                    self.port = EMAIL_TEMPLATES[i][2]\
+                        .get('port', self.port)
+                if not secure_mode:
+                    self.secure_mode = EMAIL_TEMPLATES[i][2]\
+                        .get('secure_mode', self.secure_mode)
 
                 # Adjust email login based on the defined usertype. If no entry
                 # was specified, then we default to having them all set (which
@@ -708,8 +727,8 @@ class NotifyEmail(NotifyBase):
                             attachment.url(privacy=True)))
 
                     with open(attachment.path, "rb") as abody:
-                        app = MIMEApplication(
-                            abody.read(), attachment.mimetype)
+                        app = MIMEApplication(abody.read())
+                        app.set_type(attachment.mimetype)
 
                         app.add_header(
                             'Content-Disposition',
@@ -887,13 +906,6 @@ class NotifyEmail(NotifyBase):
             # Extract from name to associate with from address
             results['from_name'] = NotifyEmail.unquote(results['qsd']['name'])
 
-        if 'timeout' in results['qsd'] and len(results['qsd']['timeout']):
-            # Deprecated in favor of cto= flag
-            NotifyBase.logger.deprecate(
-                "timeout= argument is deprecated; use cto= instead.")
-            results['qsd']['cto'] = results['qsd']['timeout']
-            del results['qsd']['timeout']
-
         # Store SMTP Host if specified
         if 'smtp' in results['qsd'] and len(results['qsd']['smtp']):
             # Extract the smtp server
