@@ -30,6 +30,7 @@
 import re
 import six
 import requests
+from markdown import markdown
 from json import dumps
 from json import loads
 from time import time
@@ -41,6 +42,7 @@ from ..common import NotifyImageSize
 from ..common import NotifyFormat
 from ..utils import parse_bool
 from ..utils import parse_list
+from ..utils import is_hostname
 from ..utils import validate_regex
 from ..AppriseLocale import gettext_lazy as _
 
@@ -65,6 +67,21 @@ IS_ROOM_ID = re.compile(
     r'(?P<home_server>[a-z0-9.-]+))?\s*$', re.I)
 
 
+class MatrixMessageType(object):
+    """
+    The Matrix Message types
+    """
+    TEXT = "text"
+    NOTICE = "notice"
+
+
+# matrix message types are placed into this list for validation purposes
+MATRIX_MESSAGE_TYPES = (
+    MatrixMessageType.TEXT,
+    MatrixMessageType.NOTICE,
+)
+
+
 class MatrixWebhookMode(object):
     # Webhook Mode is disabled
     DISABLED = "off"
@@ -79,7 +96,7 @@ class MatrixWebhookMode(object):
     T2BOT = "t2bot"
 
 
-# webhook modes are placed ito this list for validation purposes
+# webhook modes are placed into this list for validation purposes
 MATRIX_WEBHOOK_MODES = (
     MatrixWebhookMode.DISABLED,
     MatrixWebhookMode.MATRIX,
@@ -131,13 +148,13 @@ class NotifyMatrix(NotifyBase):
         '{schema}://{token}',
         '{schema}://{user}@{token}',
 
-        # All other non-t2bot setups require targets
+        # Disabled webhook
         '{schema}://{user}:{password}@{host}/{targets}',
         '{schema}://{user}:{password}@{host}:{port}/{targets}',
-        '{schema}://{token}:{password}@{host}/{targets}',
-        '{schema}://{token}:{password}@{host}:{port}/{targets}',
-        '{schema}://{user}:{token}:{password}@{host}/{targets}',
-        '{schema}://{user}:{token}:{password}@{host}:{port}/{targets}',
+
+        # Webhook mode
+        '{schema}://{user}:{token}@{host}/{targets}',
+        '{schema}://{user}:{token}@{host}:{port}/{targets}',
     )
 
     # Define our template tokens
@@ -204,12 +221,22 @@ class NotifyMatrix(NotifyBase):
             'values': MATRIX_WEBHOOK_MODES,
             'default': MatrixWebhookMode.DISABLED,
         },
+        'msgtype': {
+            'name': _('Message Type'),
+            'type': 'choice:string',
+            'values': MATRIX_MESSAGE_TYPES,
+            'default': MatrixMessageType.TEXT,
+        },
         'to': {
             'alias_of': 'targets',
         },
+        'token': {
+            'alias_of': 'token',
+        },
     })
 
-    def __init__(self, targets=None, mode=None, include_image=False, **kwargs):
+    def __init__(self, targets=None, mode=None, msgtype=None,
+                 include_image=False, **kwargs):
         """
         Initialize Matrix Object
         """
@@ -235,20 +262,44 @@ class NotifyMatrix(NotifyBase):
         self._room_cache = {}
 
         # Setup our mode
-        self.mode = MatrixWebhookMode.DISABLED \
+        self.mode = self.template_args['mode']['default'] \
             if not isinstance(mode, six.string_types) else mode.lower()
         if self.mode and self.mode not in MATRIX_WEBHOOK_MODES:
             msg = 'The mode specified ({}) is invalid.'.format(mode)
             self.logger.warning(msg)
             raise TypeError(msg)
 
+        # Setup our message type
+        self.msgtype = self.template_args['msgtype']['default'] \
+            if not isinstance(msgtype, six.string_types) else msgtype.lower()
+        if self.msgtype and self.msgtype not in MATRIX_MESSAGE_TYPES:
+            msg = 'The msgtype specified ({}) is invalid.'.format(msgtype)
+            self.logger.warning(msg)
+            raise TypeError(msg)
+
         if self.mode == MatrixWebhookMode.T2BOT:
             # t2bot configuration requires that a webhook id is specified
             self.access_token = validate_regex(
-                self.host, r'^[a-z0-9]{64}$', 'i')
+                self.password, r'^[a-z0-9]{64}$', 'i')
             if not self.access_token:
                 msg = 'An invalid T2Bot/Matrix Webhook ID ' \
-                      '({}) was specified.'.format(self.host)
+                      '({}) was specified.'.format(self.password)
+                self.logger.warning(msg)
+                raise TypeError(msg)
+
+        elif not is_hostname(self.host):
+            msg = 'An invalid Matrix Hostname ({}) was specified'\
+                  .format(self.host)
+            self.logger.warning(msg)
+            raise TypeError(msg)
+        else:
+            # Verify port if specified
+            if self.port is not None and not (
+                    isinstance(self.port, int)
+                    and self.port >= self.template_tokens['port']['min']
+                    and self.port <= self.template_tokens['port']['max']):
+                msg = 'An invalid Matrix Port ({}) was specified'\
+                      .format(self.port)
                 self.logger.warning(msg)
                 raise TypeError(msg)
 
@@ -283,7 +334,7 @@ class NotifyMatrix(NotifyBase):
             default_port = 443 if self.secure else 80
 
             # Prepare our URL
-            url = '{schema}://{hostname}:{port}/{webhook_path}/{token}'.format(
+            url = '{schema}://{hostname}:{port}{webhook_path}/{token}'.format(
                 schema='https' if self.secure else 'http',
                 hostname=self.host,
                 port='' if self.port is None
@@ -412,20 +463,26 @@ class NotifyMatrix(NotifyBase):
         payload = {
             'displayName':
                 self.user if self.user else self.app_id,
-            'format': 'html',
+            'format': 'plain' if self.notify_format == NotifyFormat.TEXT
+            else 'html',
+            'text': '',
         }
 
         if self.notify_format == NotifyFormat.HTML:
-            payload['text'] = '{}{}'.format('' if not title else title, body)
+            payload['text'] = '{title}{body}'.format(
+                title='' if not title else '<h1>{}</h1>'.format(
+                    NotifyMatrix.escape_html(title)),
+                body=body)
 
-        else:  # TEXT or MARKDOWN
+        elif self.notify_format == NotifyFormat.MARKDOWN:
+            payload['text'] = '{title}{body}'.format(
+                title='' if not title else '<h1>{}</h1>'.format(
+                    NotifyMatrix.escape_html(title)),
+                body=markdown(body))
 
-            # Ensure our content is escaped
-            title = NotifyMatrix.escape_html(title)
-            body = NotifyMatrix.escape_html(body)
-
-            payload['text'] = '{}{}'.format(
-                '' if not title else '<h4>{}</h4>'.format(title), body)
+        else:  # NotifyFormat.TEXT
+            payload['text'] = \
+                body if not title else '{}\r\n{}'.format(title, body)
 
         return payload
 
@@ -494,11 +551,6 @@ class NotifyMatrix(NotifyBase):
                 has_error = True
                 continue
 
-            # We have our data cached at this point we can freely use it
-            msg = '{title}{body}'.format(
-                title='' if not title else '{}\r\n'.format(title),
-                body=body)
-
             # Acquire our image url if we're configured to do so
             image_url = None if not self.include_image else \
                 self.image_url(notify_type)
@@ -523,10 +575,33 @@ class NotifyMatrix(NotifyBase):
 
             # Define our payload
             payload = {
-                'msgtype': 'm.text',
-                'body': msg,
+                'msgtype': 'm.{}'.format(self.msgtype),
+                'body': '{title}{body}'.format(
+                    title='' if not title else '# {}\r\n'.format(title),
+                    body=body),
             }
 
+            # Update our payload advance formatting for the services that
+            # support them.
+            if self.notify_format == NotifyFormat.HTML:
+                payload.update({
+                    'format': 'org.matrix.custom.html',
+                    'formatted_body': '{title}{body}'.format(
+                        title='' if not title else '<h1>{}</h1>'.format(title),
+                        body=body,
+                    )
+                })
+
+            elif self.notify_format == NotifyFormat.MARKDOWN:
+                payload.update({
+                    'format': 'org.matrix.custom.html',
+                    'formatted_body': '{title}{body}'.format(
+                        title='' if not title else '<h1>{}</h1>'.format(
+                            NotifyMatrix.escape_html(title, whitespace=False)),
+                        body=markdown(body),
+                    )
+                })
+
             # Build our path
             path = '/rooms/{}/send/m.room.message'.format(
                 NotifyMatrix.quote(room_id))
@@ -694,7 +769,7 @@ class NotifyMatrix(NotifyBase):
         # Prepare our Join Payload
         payload = {}
 
-        # Not in cache, next step is to check if it's a room id...
+        # Check if it's a room id...
         result = IS_ROOM_ID.match(room)
         if result:
             # We detected ourselves the home_server
@@ -707,11 +782,23 @@ class NotifyMatrix(NotifyBase):
                 home_server,
             )
 
+            # Check our cache for speed:
+            if room_id in self._room_cache:
+                # We're done as we've already joined the channel
+                return self._room_cache[room_id]['id']
+
             # Build our URL
             path = '/join/{}'.format(NotifyMatrix.quote(room_id))
 
             # Make our query
             postokay, _ = self._fetch(path, payload=payload)
+            if postokay:
+                # Cache our entry for fast access later
+                self._room_cache[room_id] = {
+                    'id': room_id,
+                    'home_server': home_server,
+                }
+
             return room_id if postokay else None
 
         # Try to see if it's an alias then...
@@ -1003,9 +1090,54 @@ class NotifyMatrix(NotifyBase):
         """
         Ensure we relinquish our token
         """
-        if self.mode != MatrixWebhookMode.T2BOT:
+        if self.mode == MatrixWebhookMode.T2BOT:
+            # nothing to do
+            return
+
+        try:
             self._logout()
 
+        except LookupError:  # pragma: no cover
+            # Python v3.5 call to requests can sometimes throw the exception
+            #   "/usr/lib64/python3.7/socket.py", line 748, in getaddrinfo
+            #   LookupError: unknown encoding: idna
+            #
+            # This occurs every time when running unit-tests against Apprise:
+            # LANG=C.UTF-8 PYTHONPATH=$(pwd) py.test-3.7
+            #
+            # There has been an open issue on this since Jan 2017.
+            #   - https://bugs.python.org/issue29288
+            #
+            # A ~similar~ issue can be identified here in the requests
+            # ticket system as unresolved and has provided work-arounds
+            #   - https://github.com/kennethreitz/requests/issues/3578
+            pass
+
+        except ImportError:  # pragma: no cover
+            # The actual exception is `ModuleNotFoundError` however ImportError
+            # grants us backwards compatiblity with versions of Python older
+            # than v3.6
+
+            # Python code that makes early calls to sys.exit() can cause
+            # the __del__() code to run. However in some newer versions of
+            # Python, this causes the `sys` library to no longer be
+            # available. The stack overflow also goes on to suggest that
+            # it's not wise to use the __del__() as a deconstructor
+            # which is the case here.
+
+            # https://stackoverflow.com/questions/67218341/\
+            #       modulenotfounderror-import-of-time-halted-none-in-sys-\
+            #           modules-occured-when-obj?noredirect=1&lq=1
+            #
+            #
+            # Also see: https://stackoverflow.com/questions\
+            #       /1481488/what-is-the-del-method-and-how-do-i-call-it
+
+            # At this time it seems clean to try to log out (if we can)
+            # but not throw any unessisary exceptions (like this one) to
+            # the end user if we don't have to.
+            pass
+
     def url(self, privacy=False, *args, **kwargs):
         """
         Returns the URL built dynamically based on specified arguments.
@@ -1015,31 +1147,36 @@ class NotifyMatrix(NotifyBase):
         params = {
             'image': 'yes' if self.include_image else 'no',
             'mode': self.mode,
+            'msgtype': self.msgtype,
         }
 
         # Extend our parameters
         params.update(self.url_parameters(privacy=privacy, *args, **kwargs))
 
-        # Determine Authentication
         auth = ''
-        if self.user and self.password:
-            auth = '{user}:{password}@'.format(
-                user=NotifyMatrix.quote(self.user, safe=''),
-                password=self.pprint(
-                    self.password, privacy, mode=PrivacyMode.Secret, safe=''),
-            )
+        if self.mode != MatrixWebhookMode.T2BOT:
+            # Determine Authentication
+            if self.user and self.password:
+                auth = '{user}:{password}@'.format(
+                    user=NotifyMatrix.quote(self.user, safe=''),
+                    password=self.pprint(
+                        self.password, privacy, mode=PrivacyMode.Secret,
+                        safe=''),
+                )
 
-        elif self.user:
-            auth = '{user}@'.format(
-                user=NotifyMatrix.quote(self.user, safe=''),
-            )
+            elif self.user:
+                auth = '{user}@'.format(
+                    user=NotifyMatrix.quote(self.user, safe=''),
+                )
 
         default_port = 443 if self.secure else 80
 
         return '{schema}://{auth}{hostname}{port}/{rooms}?{params}'.format(
             schema=self.secure_protocol if self.secure else self.protocol,
             auth=auth,
-            hostname=NotifyMatrix.quote(self.host, safe=''),
+            hostname=NotifyMatrix.quote(self.host, safe='')
+            if self.mode != MatrixWebhookMode.T2BOT
+            else self.pprint(self.access_token, privacy, safe=''),
             port='' if self.port is None
             or self.port == default_port else ':{}'.format(self.port),
             rooms=NotifyMatrix.quote('/'.join(self.rooms)),
@@ -1086,6 +1223,20 @@ class NotifyMatrix(NotifyBase):
             # Default mode to t2bot
             results['mode'] = MatrixWebhookMode.T2BOT
 
+        if results['mode'] and \
+                results['mode'].lower() == MatrixWebhookMode.T2BOT:
+            # unquote our hostname and pass it in as the password/token
+            results['password'] = NotifyMatrix.unquote(results['host'])
+
+        # Support the message type keyword
+        if 'msgtype' in results['qsd'] and len(results['qsd']['msgtype']):
+            results['msgtype'] = \
+                NotifyMatrix.unquote(results['qsd']['msgtype'])
+
+        # Support the use of the token= keyword
+        if 'token' in results['qsd'] and len(results['qsd']['token']):
+            results['password'] = NotifyMatrix.unquote(results['qsd']['token'])
+
         return results
 
     @staticmethod
