@@ -93,6 +93,9 @@ class NotifyTelegram(NotifyBase):
     # A URL that takes you to the setup/help of the specific protocol
     setup_url = 'https://github.com/caronc/apprise/wiki/Notify_telegram'
 
+    # Default Notify Format
+    notify_format = NotifyFormat.HTML
+
     # Telegram uses the http protocol with JSON requests
     notify_url = 'https://api.telegram.org/bot'
 
@@ -102,6 +105,9 @@ class NotifyTelegram(NotifyBase):
     # The maximum allowable characters allowed in the body per message
     body_maxlen = 4096
 
+    # Title is to be part of body
+    title_maxlen = 0
+
     # Telegram is limited to sending a maximum of 100 requests per second.
     request_rate_per_sec = 0.001
 
@@ -167,6 +173,90 @@ class NotifyTelegram(NotifyBase):
         },
     )
 
+    # Telegram's HTML support doesn't like having HTML escaped
+    # characters passed into it.  to handle this situation, we need to
+    # search the body for these sequences and convert them to the
+    # output the user expected
+    __telegram_escape_html_entries = (
+        # Comments
+        (re.compile(
+            r'\s*<!.+?-->\s*',
+            (re.I | re.M | re.S)), '', {}),
+
+        # the following tags are not supported
+        (re.compile(
+            r'\s*<\s*(!?DOCTYPE|p|div|span|body|script|link|'
+            r'meta|html|font|head|label|form|input|textarea|select|iframe|'
+            r'source|script)([^a-z0-9>][^>]*)?>\s*',
+            (re.I | re.M | re.S)), '', {}),
+
+        # All closing tags to be removed are put here
+        (re.compile(
+            r'\s*<\s*/(span|body|script|meta|html|font|head|'
+            r'label|form|input|textarea|select|ol|ul|link|'
+            r'iframe|source|script)([^a-z0-9>][^>]*)?>\s*',
+            (re.I | re.M | re.S)), '', {}),
+
+        # Bold
+        (re.compile(
+            r'<\s*(strong)([^a-z0-9>][^>]*)?>',
+            (re.I | re.M | re.S)), '<b>', {}),
+        (re.compile(
+            r'<\s*/\s*(strong)([^a-z0-9>][^>]*)?>',
+            (re.I | re.M | re.S)), '</b>', {}),
+        (re.compile(
+            r'\s*<\s*(h[1-6]|title)([^a-z0-9>][^>]*)?>\s*',
+            (re.I | re.M | re.S)), '{}<b>', {'html': '\r\n'}),
+        (re.compile(
+            r'\s*<\s*/\s*(h[1-6]|title)([^a-z0-9>][^>]*)?>\s*',
+            (re.I | re.M | re.S)),
+            '</b>{}', {'html': '<br/>'}),
+
+        # Italic
+        (re.compile(
+            r'<\s*(caption|em)([^a-z0-9>][^>]*)?>',
+            (re.I | re.M | re.S)), '<i>', {}),
+        (re.compile(
+            r'<\s*/\s*(caption|em)([^a-z0-9>][^>]*)?>',
+            (re.I | re.M | re.S)), '</i>', {}),
+
+        # Bullet Lists
+        (re.compile(
+            r'<\s*li([^a-z0-9>][^>]*)?>\s*',
+            (re.I | re.M | re.S)), ' -', {}),
+
+        # convert pre tags to code (supported by Telegram)
+        (re.compile(
+            r'<\s*pre([^a-z0-9>][^>]*)?>',
+            (re.I | re.M | re.S)), '{}<code>', {'html': '\r\n'}),
+        (re.compile(
+            r'<\s*/\s*pre([^a-z0-9>][^>]*)?>',
+            (re.I | re.M | re.S)), '</code>{}', {'html': '\r\n'}),
+
+        # New Lines
+        (re.compile(
+            r'\s*<\s*/?\s*(ol|ul|br|hr)\s*/?>\s*',
+            (re.I | re.M | re.S)), '\r\n', {}),
+        (re.compile(
+            r'\s*<\s*/\s*(br|p|hr|li|div)([^a-z0-9>][^>]*)?>\s*',
+            (re.I | re.M | re.S)), '\r\n', {}),
+
+        # HTML Spaces (&nbsp;) and tabs (&emsp;) aren't supported
+        # See https://core.telegram.org/bots/api#html-style
+        (re.compile(r'\&nbsp;?', re.I), ' ', {}),
+
+        # Tabs become 3 spaces
+        (re.compile(r'\&emsp;?', re.I), '   ', {}),
+
+        # Some characters get re-escaped by the Telegram upstream
+        # service so we need to convert these back,
+        (re.compile(r'\&apos;?', re.I), '\'', {}),
+        (re.compile(r'\&quot;?', re.I), '"', {}),
+
+        # New line cleanup
+        (re.compile(r'\r*\n[\r\n]+', re.I), '\r\n', {}),
+    )
+
     # Define our template tokens
     template_tokens = dict(NotifyBase.template_tokens, **{
         'bot_token': {
@@ -483,15 +573,15 @@ class NotifyTelegram(NotifyBase):
         #      "text":"/start",
         #      "entities":[{"offset":0,"length":6,"type":"bot_command"}]}}]
 
-        if 'ok' in response and response['ok'] is True \
-                and 'result' in response and len(response['result']):
-            entry = response['result'][0]
-            _id = entry['message']['from'].get('id', 0)
-            _user = entry['message']['from'].get('first_name')
-            self.logger.info('Detected Telegram user %s (userid=%d)' % (
-                _user, _id))
-            # Return our detected userid
-            return _id
+        if response.get('ok', False):
+            for entry in response.get('result', []):
+                if 'message' in entry and 'from' in entry['message']:
+                    _id = entry['message']['from'].get('id', 0)
+                    _user = entry['message']['from'].get('first_name')
+                    self.logger.info(
+                        'Detected Telegram user %s (userid=%d)' % (_user, _id))
+                    # Return our detected userid
+                    return _id
 
         self.logger.warning(
             'Failed to detect a Telegram user; '
@@ -499,7 +589,7 @@ class NotifyTelegram(NotifyBase):
         return 0
 
     def send(self, body, title='', notify_type=NotifyType.INFO, attach=None,
-             **kwargs):
+             body_format=None, **kwargs):
         """
         Perform Telegram Notification
         """
@@ -538,87 +628,28 @@ class NotifyTelegram(NotifyBase):
             'disable_web_page_preview': not self.preview,
         }
 
-        # Prepare Email Message
+        # Prepare Message Body
         if self.notify_format == NotifyFormat.MARKDOWN:
             payload['parse_mode'] = 'MARKDOWN'
 
-            payload['text'] = '{}{}'.format(
-                '{}\r\n'.format(title) if title else '',
-                body,
-            )
+            payload['text'] = body
 
-        else:  # HTML or TEXT
+        else:  # HTML
 
             # Use Telegram's HTML mode
             payload['parse_mode'] = 'HTML'
+            for r, v, m in self.__telegram_escape_html_entries:
 
-            # Telegram's HTML support doesn't like having HTML escaped
-            # characters passed into it.  to handle this situation, we need to
-            # search the body for these sequences and convert them to the
-            # output the user expected
-            telegram_escape_html_dict = {
-                # HTML Spaces (&nbsp;) and tabs (&emsp;) aren't supported
-                # See https://core.telegram.org/bots/api#html-style
-                r'nbsp': ' ',
+                if 'html' in m:
+                    # Handle special cases where we need to alter new lines
+                    # for presentation purposes
+                    v = v.format(m['html'] if body_format in (
+                        NotifyFormat.HTML, NotifyFormat.MARKDOWN) else '')
 
-                # Tabs become 3 spaces
-                r'emsp': '   ',
+                body = r.sub(v, body)
 
-                # Some characters get re-escaped by the Telegram upstream
-                # service so we need to convert these back,
-                r'apos': '\'',
-                r'quot': '"',
-            }
-
-            # Create a regular expression from the dictionary keys
-            html_regex = re.compile("&(%s);?" % "|".join(
-                map(re.escape, telegram_escape_html_dict.keys())).lower(),
-                re.I)
-
-            # For each match, look-up corresponding value in dictionary
-            # we look +1 to ignore the & that does not appear in the index
-            # we only look at the first 4 characters because we don't want to
-            # fail on &apos; as it's accepted (along with &apos - no
-            # semi-colon)
-            body = html_regex.sub(  # pragma: no branch
-                lambda mo: telegram_escape_html_dict[
-                    mo.string[mo.start():mo.end()][1:5]], body)
-
-            if title:
-                # For each match, look-up corresponding value in dictionary
-                # Indexing is explained above (for how the body is parsed)
-                title = html_regex.sub(  # pragma: no branch
-                    lambda mo: telegram_escape_html_dict[
-                        mo.string[mo.start():mo.end()][1:5]], title)
-
-            if self.notify_format == NotifyFormat.TEXT:
-                telegram_escape_text_dict = {
-                    # We need to escape characters that conflict with html
-                    # entity blocks (< and >) when displaying text
-                    r'>': '&gt;',
-                    r'<': '&lt;',
-                }
-
-                # Create a regular expression from the dictionary keys
-                text_regex = re.compile("(%s)" % "|".join(
-                    map(re.escape, telegram_escape_text_dict.keys())).lower(),
-                    re.I)
-
-                # For each match, look-up corresponding value in dictionary
-                body = text_regex.sub(  # pragma: no branch
-                    lambda mo: telegram_escape_text_dict[
-                        mo.string[mo.start():mo.end()]], body)
-
-                if title:
-                    # For each match, look-up corresponding value in dictionary
-                    title = text_regex.sub(  # pragma: no branch
-                        lambda mo: telegram_escape_text_dict[
-                            mo.string[mo.start():mo.end()]], title)
-
-            payload['text'] = '{}{}'.format(
-                '<b>{}</b>\r\n'.format(title) if title else '',
-                body,
-            )
+            # Prepare our payload based on HTML or TEXT
+            payload['text'] = body
 
         # Create a copy of the chat_ids list
         targets = list(self.targets)
