@@ -25,23 +25,78 @@
 
 import re
 import six
+import sys
 import json
 import contextlib
 import os
+import hashlib
+from itertools import chain
 from os.path import expanduser
 from functools import reduce
+from . import common
+from .logger import logger
 
 try:
     # Python 2.7
     from urllib import unquote
     from urllib import quote
     from urlparse import urlparse
+    from urllib import urlencode as _urlencode
+
+    import imp
+
+    def import_module(path, name):
+        """
+        Load our module based on path
+        """
+        try:
+            return imp.load_source(name, path)
+
+        except Exception as e:
+            logger.debug(
+                'Custom module exception raised from %s (name=%s) %s',
+                path, name, str(e))
+
+        return None
 
 except ImportError:
-    # Python 3.x
+    # Python 3.5+
     from urllib.parse import unquote
     from urllib.parse import quote
     from urllib.parse import urlparse
+    from urllib.parse import urlencode as _urlencode
+
+    import importlib.util
+
+    def import_module(path, name):
+        """
+        Load our module based on path
+        """
+        # if path.endswith('test_module_detection0/a/hook.py'):
+        #     import pdb
+        #     pdb.set_trace()
+
+        spec = importlib.util.spec_from_file_location(name, path)
+        try:
+            module = importlib.util.module_from_spec(spec)
+            sys.modules[name] = module
+
+            spec.loader.exec_module(module)
+
+        except Exception as e:
+            # module isn't loadable
+            del sys.modules[name]
+            module = None
+
+            logger.debug(
+                'Custom module exception raised from %s (name=%s) %s',
+                path, name, str(e))
+
+        return module
+
+# Hash of all paths previously scanned so we don't waste effort/overhead doing
+# it again
+PATHS_PREVIOUSLY_SCANNED = set()
 
 # URL Indexing Table for returns via parse_url()
 # The below accepts and scans for:
@@ -104,6 +159,13 @@ NOTIFY_CUSTOM_COLON_TOKENS = re.compile(r'^:(?P<key>.*)\s*')
 # Used for attempting to acquire the schema if the URL can't be parsed.
 GET_SCHEMA_RE = re.compile(r'\s*(?P<schema>[a-z0-9]{2,9})://.*$', re.I)
 
+# Used for validating that a provided entry is indeed a schema
+# this is slightly different then the GET_SCHEMA_RE above which
+# insists the schema is only valid with a :// entry.  this one
+# extrapolates the individual entries
+URL_DETAILS_RE = re.compile(
+    r'\s*(?P<schema>[a-z0-9]{2,9})(://(?P<base>.*))?$', re.I)
+
 # Regular expression based and expanded from:
 # http://www.regular-expressions.info/email.html
 # Extended to support colon (:) delimiter for parsing names from the URL
@@ -133,6 +195,17 @@ IS_PHONE_NO = re.compile(r'^\+?(?P<phone>[0-9\s)(+-]+)\s*$')
 PHONE_NO_DETECTION_RE = re.compile(
     r'\s*([+(\s]*[0-9][0-9()\s-]+[0-9])(?=$|[\s,+(]+[0-9])', re.I)
 
+# A simple verification check to make sure the content specified
+# rougly conforms to a ham radio call sign before we parse it further
+IS_CALL_SIGN = re.compile(
+    r'^(?P<callsign>[a-z0-9]{2,3}[0-9][a-z0-9]{3})'
+    r'(?P<ssid>-[a-z0-9]{1,2})?\s*$', re.I)
+
+# Regular expression used to destinguish between multiple ham radio call signs
+CALL_SIGN_DETECTION_RE = re.compile(
+    r'\s*([a-z0-9]{2,3}[0-9][a-z0-9]{3}(?:-[a-z0-9]{1,2})?)'
+    r'(?=$|[\s,]+[a-z0-9]{4,6})', re.I)
+
 # Regular expression used to destinguish between multiple URLs
 URL_DETECTION_RE = re.compile(
     r'([a-z0-9]+?:\/\/.*?)(?=$|[\s,]+[a-z0-9]{2,9}?:\/\/)', re.I)
@@ -280,7 +353,7 @@ def is_uuid(uuid):
     return True if match else False
 
 
-def is_phone_no(phone, min_len=11):
+def is_phone_no(phone, min_len=10):
     """Determine if the specified entry is a phone number
 
     Args:
@@ -372,6 +445,37 @@ def is_phone_no(phone, min_len=11):
     }
 
 
+def is_call_sign(callsign):
+    """Determine if the specified entry is a ham radio call sign
+
+    Args:
+        callsign (str): The string you want to check.
+
+    Returns:
+        bool: Returns False if the address specified is not a phone number
+    """
+
+    try:
+        result = IS_CALL_SIGN.match(callsign)
+        if not result:
+            # not parseable content as it does not even conform closely to a
+            # callsign
+            return False
+
+    except TypeError:
+        # not parseable content
+        return False
+
+    ssid = result.group('ssid')
+    return {
+        # always treat call signs as uppercase content
+        'callsign': result.group('callsign').upper(),
+        # Prevent the storing of the None keyword in the event the SSID was
+        # not detected
+        'ssid': ssid if ssid else '',
+    }
+
+
 def is_email(address):
     """Determine if the specified entry is an email address
 
@@ -432,16 +536,14 @@ def tidy_path(path):
     # Windows
     path = TIDY_WIN_PATH_RE.sub('\\1', path.strip())
     # Linux
-    path = TIDY_NUX_PATH_RE.sub('\\1', path.strip())
+    path = TIDY_NUX_PATH_RE.sub('\\1', path)
 
-    # Linux Based Trim
-    path = TIDY_NUX_TRIM_RE.sub('\\1', path.strip())
-    # Windows Based Trim
-    path = expanduser(TIDY_WIN_TRIM_RE.sub('\\1', path.strip()))
+    # Windows Based (final) Trim
+    path = expanduser(TIDY_WIN_TRIM_RE.sub('\\1', path))
     return path
 
 
-def parse_qsd(qs):
+def parse_qsd(qs, simple=False):
     """
     Query String Dictionary Builder
 
@@ -460,6 +562,9 @@ def parse_qsd(qs):
 
     This function returns a result object that fits with the apprise
     expected parameters (populating the 'qsd' portion of the dictionary
+
+    if simple is set to true, then a ONE dictionary is returned and is not
+    sub-parsed for additional elements
     """
 
     # Our return result set:
@@ -475,7 +580,7 @@ def parse_qsd(qs):
         'qsd+': {},
         'qsd-': {},
         'qsd:': {},
-    }
+    } if not simple else {'qsd': {}}
 
     pairs = [s2 for s1 in qs.split('&') for s2 in s1.split(';')]
     for name_value in pairs:
@@ -502,6 +607,10 @@ def parse_qsd(qs):
         # content is always made lowercase for easy indexing
         result['qsd'][key.lower().strip()] = val
 
+        if simple:
+            # move along
+            continue
+
         # Check for tokens that start with a addition/plus symbol (+)
         k = NOTIFY_CUSTOM_ADD_TOKENS.match(key)
         if k is not None:
@@ -523,7 +632,8 @@ def parse_qsd(qs):
     return result
 
 
-def parse_url(url, default_schema='http', verify_host=True):
+def parse_url(url, default_schema='http', verify_host=True, strict_port=False,
+              simple=False):
     """A function that greatly simplifies the parsing of a url
     specified by the end user.
 
@@ -545,6 +655,39 @@ def parse_url(url, default_schema='http', verify_host=True):
      The function returns a simple dictionary with all of
      the parsed content within it and returns 'None' if the
      content could not be extracted.
+
+     The output of 'http://hostname' would look like:
+        {
+          'schema': 'http',
+          'url': 'http://hostname',
+          'host': 'hostname',
+
+          'user': None,
+          'password': None,
+          'port': None,
+          'fullpath': None,
+          'path': None,
+          'query': None,
+
+          'qsd': {},
+
+          'qsd+': {},
+          'qsd-': {},
+          'qsd:': {}
+        }
+
+     The simple switch cleans the dictionary response to only include the
+     fields that were detected.
+
+     The output of 'http://hostname' with the simple flag set would look like:
+        {
+          'schema': 'http',
+          'url': 'http://hostname',
+          'host': 'hostname',
+        }
+
+     If the URL can't be parsed then None is returned
+
     """
 
     if not isinstance(url, six.string_types):
@@ -583,7 +726,7 @@ def parse_url(url, default_schema='http', verify_host=True):
         'qsd+': {},
         'qsd-': {},
         'qsd:': {},
-    }
+    } if not simple else {}
 
     qsdata = ''
     match = VALID_URL_RE.search(url)
@@ -603,7 +746,7 @@ def parse_url(url, default_schema='http', verify_host=True):
     # Parse Query Arugments ?val=key&key=val
     # while ensuring that all keys are lowercase
     if qsdata:
-        result.update(parse_qsd(qsdata))
+        result.update(parse_qsd(qsdata, simple=simple))
 
     # Now do a proper extraction of data; http:// is just substitued in place
     # to allow urlparse() to function as expected, we'll swap this back to the
@@ -626,8 +769,12 @@ def parse_url(url, default_schema='http', verify_host=True):
         pass
 
     if not result['fullpath']:
-        # Default
-        result['fullpath'] = None
+        if not simple:
+            # Default
+            result['fullpath'] = None
+        else:
+            # Remove entry
+            del result['fullpath']
 
     else:
         # Using full path, extract query from path
@@ -635,7 +782,11 @@ def parse_url(url, default_schema='http', verify_host=True):
         result['path'] = match.group('path')
         result['query'] = match.group('query')
         if not result['query']:
-            result['query'] = None
+            if not simple:
+                result['query'] = None
+            else:
+                del result['query']
+
     try:
         (result['user'], result['host']) = \
             re.split(r'[@]+', result['host'])[:2]
@@ -645,7 +796,7 @@ def parse_url(url, default_schema='http', verify_host=True):
         # and it's already assigned
         pass
 
-    if result['user'] is not None:
+    if result.get('user') is not None:
         try:
             (result['user'], result['password']) = \
                 re.split(r'[:]+', result['user'])[:2]
@@ -655,13 +806,32 @@ def parse_url(url, default_schema='http', verify_host=True):
             # and it's already assigned
             pass
 
-    # Max port is 65535 so (1,5 digits)
-    match = re.search(
-        r'^(?P<host>.+):(?P<port>[1-9][0-9]{0,4})$', result['host'])
-    if match:
+    # Port Parsing
+    pmatch = re.search(
+        r'^(?P<host>(\[[0-9a-f:]+\]|[^:]+)):(?P<port>[^:]*)$',
+        result['host'])
+
+    if pmatch:
         # Separate our port from our hostname (if port is detected)
-        result['host'] = match.group('host')
-        result['port'] = int(match.group('port'))
+        result['host'] = pmatch.group('host')
+        try:
+            # If we're dealing with an integer, go ahead and convert it
+            # otherwise return an 'x' which will raise a ValueError
+            #
+            # This small extra check allows us to treat floats/doubles
+            # as strings. Hence a value like '4.2' won't be converted to a 4
+            # (and the .2 lost)
+            result['port'] = int(
+                pmatch.group('port')
+                if re.search(r'[0-9]', pmatch.group('port')) else 'x')
+
+        except ValueError:
+            if verify_host:
+                # Invalid Host Specified
+                return None
+
+    # Acquire our port (if defined)
+    _port = result.get('port')
 
     if verify_host:
         # Verify and Validate our hostname
@@ -671,24 +841,55 @@ def parse_url(url, default_schema='http', verify_host=True):
             # some indication as to what went wrong
             return None
 
+        # Max port is 65535 and min is 1
+        if isinstance(_port, int) and not ((
+            not strict_port or (
+                strict_port and _port > 0 and _port <= 65535))):
+
+            # An invalid port was specified
+            return None
+
+    elif pmatch and not isinstance(_port, int):
+        if strict_port:
+            # Store port
+            result['port'] = pmatch.group('port').strip()
+
+        else:
+            # Fall back
+            result['port'] = None
+            result['host'] = '{}:{}'.format(
+                pmatch.group('host'), pmatch.group('port'))
+
     # Re-assemble cleaned up version of the url
     result['url'] = '%s://' % result['schema']
-    if isinstance(result['user'], six.string_types):
+    if isinstance(result.get('user'), six.string_types):
         result['url'] += result['user']
 
-        if isinstance(result['password'], six.string_types):
+        if isinstance(result.get('password'), six.string_types):
             result['url'] += ':%s@' % result['password']
 
         else:
             result['url'] += '@'
     result['url'] += result['host']
 
-    if result['port']:
-        result['url'] += ':%d' % result['port']
+    if result.get('port') is not None:
+        try:
+            result['url'] += ':%d' % result['port']
 
-    if result['fullpath']:
+        except TypeError:
+            result['url'] += ':%s' % result['port']
+
+    elif 'port' in result and simple:
+        # Eliminate empty fields
+        del result['port']
+
+    if result.get('fullpath'):
         result['url'] += result['fullpath']
 
+    if simple and not result['host']:
+        # simple mode does not carry over empty host names
+        del result['host']
+
     return result
 
 
@@ -766,6 +967,43 @@ def parse_phone_no(*args, **kwargs):
     return result
 
 
+def parse_call_sign(*args, **kwargs):
+    """
+    Takes a string containing ham radio call signs separated by
+    comma and/or spacesand returns a list.
+    """
+
+    # for Python 2.7 support, store_unparsable is not in the url above
+    # as just parse_emails(*args, store_unparseable=True) since it is
+    # an invalid syntax.  This is the workaround to be backards compatible:
+    store_unparseable = kwargs.get('store_unparseable', True)
+
+    result = []
+    for arg in args:
+        if isinstance(arg, six.string_types) and arg:
+            _result = CALL_SIGN_DETECTION_RE.findall(arg)
+            if _result:
+                result += _result
+
+            elif not _result and store_unparseable:
+                # we had content passed into us that was lost because it was
+                # so poorly formatted that it didn't even come close to
+                # meeting the regular expression we defined. We intentially
+                # keep it as part of our result set so that parsing done
+                # at a higher level can at least report this to the end user
+                # and hopefully give them some indication as to what they
+                # may have done wrong.
+                result += \
+                    [x for x in filter(bool, re.split(STRING_DELIMITERS, arg))]
+
+        elif isinstance(arg, (set, list, tuple)):
+            # Use recursion to handle the list of call signs
+            result += parse_call_sign(
+                *arg, store_unparseable=store_unparseable)
+
+    return result
+
+
 def parse_emails(*args, **kwargs):
     """
     Takes a string containing emails separated by comma's and/or spaces and
@@ -838,6 +1076,81 @@ def parse_urls(*args, **kwargs):
     return result
 
 
+def url_assembly(**kwargs):
+    """
+    This function reverses the parse_url() function by taking in the provided
+    result set and re-assembling a URL
+
+    """
+    # Determine Authentication
+    auth = ''
+    if kwargs.get('user') is not None and \
+            kwargs.get('password') is not None:
+
+        auth = '{user}:{password}@'.format(
+            user=quote(kwargs.get('user'), safe=''),
+            password=quote(kwargs.get('password'), safe=''),
+        )
+
+    elif kwargs.get('user') is not None:
+        auth = '{user}@'.format(
+            user=quote(kwargs.get('user'), safe=''),
+        )
+
+    return '{schema}://{auth}{hostname}{port}{fullpath}{params}'.format(
+        schema='' if not kwargs.get('schema') else kwargs.get('schema'),
+        auth=auth,
+        # never encode hostname since we're expecting it to be a valid one
+        hostname='' if not kwargs.get('host') else kwargs.get('host', ''),
+        port='' if not kwargs.get('port')
+        else ':{}'.format(kwargs.get('port')),
+        fullpath=quote(kwargs.get('fullpath', ''), safe='/'),
+        params='' if not kwargs.get('qsd')
+        else '?{}'.format(urlencode(kwargs.get('qsd'))),
+    )
+
+
+def urlencode(query, doseq=False, safe='', encoding=None, errors=None):
+    """Convert a mapping object or a sequence of two-element tuples
+
+    Wrapper to Python's unquote while remaining compatible with both
+    Python 2 & 3 since the reference to this function changed between
+    versions.
+
+    The resulting string is a series of key=value pairs separated by '&'
+    characters, where both key and value are quoted using the quote()
+    function.
+
+    Note: If the dictionary entry contains an entry that is set to None
+          it is not included in the final result set. If you want to
+          pass in an empty variable, set it to an empty string.
+
+    Args:
+        query (str): The dictionary to encode
+        doseq (:obj:`bool`, optional): Handle sequences
+        safe (:obj:`str`): non-ascii characters and URI specific ones that
+            you do not wish to escape (if detected). Setting this string
+            to an empty one causes everything to be escaped.
+        encoding (:obj:`str`, optional): encoding type
+        errors (:obj:`str`, errors): how to handle invalid character found
+            in encoded string (defined by encoding)
+
+    Returns:
+        str: The escaped parameters returned as a string
+    """
+    # Tidy query by eliminating any records set to None
+    _query = {k: v for (k, v) in query.items() if v is not None}
+    try:
+        # Python v3.x
+        return _urlencode(
+            _query, doseq=doseq, safe=safe, encoding=encoding,
+            errors=errors)
+
+    except TypeError:
+        # Python v2.7
+        return _urlencode(_query)
+
+
 def parse_list(*args):
     """
     Take a string list and break it into a delimited
@@ -876,7 +1189,8 @@ def parse_list(*args):
     return sorted([x for x in filter(bool, list(set(result)))])
 
 
-def is_exclusive_match(logic, data, match_all='all'):
+def is_exclusive_match(logic, data, match_all=common.MATCH_ALL_TAG,
+                       match_always=common.MATCH_ALWAYS_TAG):
     """
 
     The data variable should always be a set of strings that the logic can be
@@ -892,6 +1206,9 @@ def is_exclusive_match(logic, data, match_all='all'):
         logic=['tagA', 'tagB']            = tagA or tagB
         logic=[('tagA', 'tagC'), 'tagB']  = (tagA and tagC) or tagB
         logic=[('tagB', 'tagC')]          = tagB and tagC
+
+    If `match_always` is not set to None, then its value is added as an 'or'
+    to all specified logic searches.
     """
 
     if isinstance(logic, six.string_types):
@@ -907,6 +1224,10 @@ def is_exclusive_match(logic, data, match_all='all'):
         # garbage input
         return False
 
+    if match_always:
+        # Add our match_always to our logic searching if secified
+        logic = chain(logic, [match_always])
+
     # Track what we match against; but by default we do not match
     # against anything
     matched = False
@@ -1249,3 +1570,161 @@ def apply_template(template, app_mode=TemplateType.RAW, **kwargs):
     # to drop the '{{' and '}}' surrounding our match so that we can
     # re-index it back into our list
     return mask_r.sub(lambda x: fn(kwargs[x.group()[2:-2].strip()]), template)
+
+
+def remove_suffix(value, suffix):
+    """
+    Removes a suffix from the end of a string.
+    """
+    return value[:-len(suffix)] if value.endswith(suffix) else value
+
+
+def module_detection(paths, cache=True):
+    """
+    Iterates over a defined path for apprise decorators to load such as
+    @notify.
+
+    """
+
+    # A simple restriction that we don't allow periods in the filename at all
+    # so it can't be hidden (Linux OS's) and it won't conflict with Python
+    # path naming.  This also prevents us from loading any python file that
+    # starts with an underscore or dash
+    # We allow __init__.py as well
+    module_re = re.compile(
+        r'^(?P<name>[_a-z0-9][a-z0-9._-]+)?(\.py)?$', re.I)
+
+    if isinstance(paths, six.string_types):
+        paths = [paths, ]
+
+    if not paths or not isinstance(paths, (tuple, list)):
+        # We're done
+        return None
+
+    def _import_module(path):
+        # Since our plugin name can conflict (as a module) with another
+        # we want to generate random strings to avoid steping on
+        # another's namespace
+        module_name = hashlib.sha1(path.encode('utf-8')).hexdigest()
+        module_pyname = "{prefix}.{name}".format(
+            prefix='apprise.custom.module', name=module_name)
+
+        if module_pyname in common.NOTIFY_CUSTOM_MODULE_MAP:
+            # First clear out existing entries
+            for schema in common.\
+                    NOTIFY_CUSTOM_MODULE_MAP[module_pyname]['notify'] \
+                    .keys():
+                # Remove any mapped modules to this file
+                del common.NOTIFY_SCHEMA_MAP[schema]
+
+            # Reset
+            del common.NOTIFY_CUSTOM_MODULE_MAP[module_pyname]
+
+        # Load our module
+        module = import_module(path, module_pyname)
+        if not module:
+            # No problem, we can't use this object
+            logger.warning('Failed to load custom module: %s', _path)
+            return None
+
+        # Print our loaded modules if any
+        if module_pyname in common.NOTIFY_CUSTOM_MODULE_MAP:
+            logger.debug(
+                'Loaded custom module: %s (name=%s)',
+                _path, module_name)
+
+            for schema, meta in common.\
+                    NOTIFY_CUSTOM_MODULE_MAP[module_pyname]['notify']\
+                    .items():
+
+                logger.info('Loaded custom notification: %s://', schema)
+        else:
+            # The code reaches here if we successfully loaded the Python
+            # module but no hooks/triggers were found. So we can safely
+            # just remove/ignore this entry
+            del sys.modules[module_pyname]
+            return None
+
+        # end of _import_module()
+        return None
+
+    for _path in paths:
+        path = os.path.abspath(os.path.expanduser(_path))
+        if (cache and path in PATHS_PREVIOUSLY_SCANNED) \
+                or not os.path.exists(path):
+            # We're done as we've already scanned this
+            continue
+
+        # Store our path as a way of hashing it has been handled
+        PATHS_PREVIOUSLY_SCANNED.add(path)
+
+        if os.path.isdir(path) and not \
+                os.path.isfile(os.path.join(path, '__init__.py')):
+
+            logger.debug('Scanning for custom plugins in: %s', path)
+            for entry in os.listdir(path):
+                re_match = module_re.match(entry)
+                if not re_match:
+                    # keep going
+                    logger.trace('Plugin Scan: Ignoring %s', entry)
+                    continue
+
+                new_path = os.path.join(path, entry)
+                if os.path.isdir(new_path):
+                    # Update our path
+                    new_path = os.path.join(path, entry, '__init__.py')
+                    if not os.path.isfile(new_path):
+                        logger.trace(
+                            'Plugin Scan: Ignoring %s',
+                            os.path.join(path, entry))
+                        continue
+
+                if not cache or \
+                        (cache and new_path not in PATHS_PREVIOUSLY_SCANNED):
+                    # Load our module
+                    _import_module(new_path)
+
+                    # Add our subdir path
+                    PATHS_PREVIOUSLY_SCANNED.add(new_path)
+        else:
+            if os.path.isdir(path):
+                # This logic is safe to apply because we already validated
+                # the directories state above; update our path
+                path = os.path.join(path, '__init__.py')
+                if cache and path in PATHS_PREVIOUSLY_SCANNED:
+                    continue
+
+                PATHS_PREVIOUSLY_SCANNED.add(path)
+
+            # directly load as is
+            re_match = module_re.match(os.path.basename(path))
+            # must be a match and must have a .py extension
+            if not re_match or not re_match.group(1):
+                # keep going
+                logger.trace('Plugin Scan: Ignoring %s', path)
+                continue
+
+            # Load our module
+            _import_module(path)
+
+        return None
+
+
+def dict_full_update(dict1, dict2):
+    """
+    Takes 2 dictionaries (dict1 and dict2) that contain sub-dictionaries and
+    gracefully merges them into dict1.
+
+    This is similar to: dict1.update(dict2) except that internal dictionaries
+    are also recursively applied.
+    """
+    def _merge(dict1, dict2):
+        for k in dict2:
+            if k in dict1 and isinstance(dict1[k], dict) \
+                    and isinstance(dict2[k], dict):
+                _merge(dict1[k], dict2[k])
+            else:
+                dict1[k] = dict2[k]
+
+    _merge(dict1, dict2)
+    return
