@@ -24,13 +24,17 @@
 # THE SOFTWARE.
 
 from __future__ import print_function
+import six
 import sys
 import pytest
 from apprise import Apprise
 from apprise import NotifyBase
 from apprise import NotifyFormat
 
-from apprise.plugins import SCHEMA_MAP
+from apprise.common import NOTIFY_SCHEMA_MAP
+
+if not six.PY2:
+    import apprise.py3compat.asyncio as py3aio
 
 # Disable logging for a cleaner testing output
 import logging
@@ -63,7 +67,7 @@ def test_apprise_asyncio_runtime_error():
             return NotifyBase.parse_url(url, verify_host=False)
 
     # Store our good notification in our schema map
-    SCHEMA_MAP['good'] = GoodNotification
+    NOTIFY_SCHEMA_MAP['good'] = GoodNotification
 
     # Create ourselves an Apprise object
     a = Apprise()
@@ -106,3 +110,54 @@ def test_apprise_asyncio_runtime_error():
     finally:
         # Restore our event loop (in the event the above test failed)
         asyncio.set_event_loop(loop)
+
+
+@pytest.mark.skipif(sys.version_info.major <= 2 or sys.version_info < (3, 7),
+                    reason="Requires Python 3.7+")
+def test_apprise_works_in_async_loop():
+    """
+    API: Apprise() can execute synchronously in an existing event loop
+
+    """
+    class GoodNotification(NotifyBase):
+        def __init__(self, **kwargs):
+            super(GoodNotification, self).__init__(
+                notify_format=NotifyFormat.HTML, **kwargs)
+
+        def url(self, **kwargs):
+            # Support URL
+            return ''
+
+        def send(self, **kwargs):
+            # Pretend everything is okay
+            return True
+
+        @staticmethod
+        def parse_url(url, *args, **kwargs):
+            # always parseable
+            return NotifyBase.parse_url(url, verify_host=False)
+
+    # Store our good notification in our schema map
+    NOTIFY_SCHEMA_MAP['good'] = GoodNotification
+
+    # Create ourselves an Apprise object
+    a = Apprise()
+
+    # Add a few entries
+    for _ in range(25):
+        a.add('good://')
+
+    # To ensure backwards compatibility, it should be possible to call
+    # asynchronous Apprise methods from code that already uses an event loop,
+    # even when using the synchronous notify() method.
+    # see https://github.com/caronc/apprise/issues/610
+    import asyncio
+
+    def try_notify():
+        a.notify(title="title", body="body")
+
+    # Convert to a coroutine to run asynchronously.
+    cor = py3aio.toasyncwrap(try_notify)
+
+    # Should execute successfully.
+    asyncio.run(cor)
