@@ -27,12 +27,15 @@ import re
 import mock
 import requests
 import json
+from inspect import cleandoc
 from os.path import dirname
 from os.path import join
 from apprise import cli
 from apprise import NotifyBase
+from apprise.common import NOTIFY_CUSTOM_MODULE_MAP
+from apprise.utils import PATHS_PREVIOUSLY_SCANNED
 from click.testing import CliRunner
-from apprise.plugins import SCHEMA_MAP
+from apprise.common import NOTIFY_SCHEMA_MAP
 from apprise.utils import environ
 from apprise.plugins import __load_matrix
 from apprise.plugins import __reset_matrix
@@ -85,8 +88,8 @@ def test_apprise_cli_nux_env(tmpdir):
             return 'bad://'
 
     # Set up our notification types
-    SCHEMA_MAP['good'] = GoodNotification
-    SCHEMA_MAP['bad'] = BadNotification
+    NOTIFY_SCHEMA_MAP['good'] = GoodNotification
+    NOTIFY_SCHEMA_MAP['bad'] = BadNotification
 
     runner = CliRunner()
     result = runner.invoke(cli.main)
@@ -476,10 +479,10 @@ def test_apprise_cli_nux_env(tmpdir):
         ])
         assert result.exit_code == 0
 
-    with mock.patch('apprise.cli.DEFAULT_SEARCH_PATHS', []):
+    with mock.patch('apprise.cli.DEFAULT_CONFIG_PATHS', []):
         with environ(APPRISE_URLS="      "):
             # An empty string is not valid and therefore not loaded so the
-            # below fails. We override the DEFAULT_SEARCH_PATHS because we
+            # below fails. We override the DEFAULT_CONFIG_PATHS because we
             # don't want to detect ones loaded on the machine running the unit
             # tests
             result = runner.invoke(cli.main, [
@@ -519,11 +522,11 @@ def test_apprise_cli_nux_env(tmpdir):
         ])
         assert result.exit_code == 0
 
-    with mock.patch('apprise.cli.DEFAULT_SEARCH_PATHS', []):
+    with mock.patch('apprise.cli.DEFAULT_CONFIG_PATHS', []):
         with environ(APPRISE_CONFIG="      "):
             # We will fail to send the notification as no path was
             # specified.
-            # We override the DEFAULT_SEARCH_PATHS because we don't
+            # We override the DEFAULT_CONFIG_PATHS because we don't
             # want to detect ones loaded on the machine running the unit tests
             result = runner.invoke(cli.main, [
                 '-b', 'my message',
@@ -598,7 +601,7 @@ def test_apprise_cli_nux_env(tmpdir):
 
 def test_apprise_cli_details(tmpdir):
     """
-    API: Apprise() Disabled Plugin States
+    CLI: --details (-l)
 
     """
 
@@ -644,7 +647,7 @@ def test_apprise_cli_details(tmpdir):
             # Pretend everything is okay (so we don't break other tests)
             return True
 
-    SCHEMA_MAP['req01'] = TestReq01Notification
+    NOTIFY_SCHEMA_MAP['req01'] = TestReq01Notification
 
     # This is a made up class that is just used to verify
     class TestReq02Notification(NotifyBase):
@@ -674,7 +677,7 @@ def test_apprise_cli_details(tmpdir):
             # Pretend everything is okay (so we don't break other tests)
             return True
 
-    SCHEMA_MAP['req02'] = TestReq02Notification
+    NOTIFY_SCHEMA_MAP['req02'] = TestReq02Notification
 
     # This is a made up class that is just used to verify
     class TestReq03Notification(NotifyBase):
@@ -700,7 +703,7 @@ def test_apprise_cli_details(tmpdir):
             # Pretend everything is okay (so we don't break other tests)
             return True
 
-    SCHEMA_MAP['req03'] = TestReq03Notification
+    NOTIFY_SCHEMA_MAP['req03'] = TestReq03Notification
 
     # This is a made up class that is just used to verify
     class TestReq04Notification(NotifyBase):
@@ -720,7 +723,7 @@ def test_apprise_cli_details(tmpdir):
             # Pretend everything is okay (so we don't break other tests)
             return True
 
-    SCHEMA_MAP['req04'] = TestReq04Notification
+    NOTIFY_SCHEMA_MAP['req04'] = TestReq04Notification
 
     # This is a made up class that is just used to verify
     class TestReq05Notification(NotifyBase):
@@ -741,7 +744,7 @@ def test_apprise_cli_details(tmpdir):
             # Pretend everything is okay (so we don't break other tests)
             return True
 
-    SCHEMA_MAP['req05'] = TestReq04Notification
+    NOTIFY_SCHEMA_MAP['req05'] = TestReq04Notification
 
     class TestDisabled01Notification(NotifyBase):
         """
@@ -763,7 +766,7 @@ def test_apprise_cli_details(tmpdir):
             # Pretend everything is okay (so we don't break other tests)
             return True
 
-    SCHEMA_MAP['na01'] = TestDisabled01Notification
+    NOTIFY_SCHEMA_MAP['na01'] = TestDisabled01Notification
 
     class TestDisabled02Notification(NotifyBase):
         """
@@ -788,7 +791,7 @@ def test_apprise_cli_details(tmpdir):
             # Pretend everything is okay (so we don't break other tests)
             return True
 
-    SCHEMA_MAP['na02'] = TestDisabled02Notification
+    NOTIFY_SCHEMA_MAP['na02'] = TestDisabled02Notification
 
     # We'll add a good notification to our list
     class TesEnabled01Notification(NotifyBase):
@@ -808,7 +811,7 @@ def test_apprise_cli_details(tmpdir):
             # Pretend everything is okay (so we don't break other tests)
             return True
 
-    SCHEMA_MAP['good'] = TesEnabled01Notification
+    NOTIFY_SCHEMA_MAP['good'] = TesEnabled01Notification
 
     # Verify that we can pass through all of our different details
     result = runner.invoke(cli.main, [
@@ -826,6 +829,371 @@ def test_apprise_cli_details(tmpdir):
     __load_matrix()
 
 
+@mock.patch('requests.post')
+def test_apprise_cli_plugin_loading(mock_post, tmpdir):
+    """
+    CLI: --plugin-path (-P)
+
+    """
+    # Prepare Mock
+    mock_post.return_value = requests.Request()
+    mock_post.return_value.status_code = requests.codes.ok
+
+    runner = CliRunner()
+
+    # Clear our working variables so they don't obstruct the next test
+    # This simulates an actual call from the CLI.  Unfortunately through
+    # testing were occupying the same memory space so our singleton's
+    # have already been populated
+    PATHS_PREVIOUSLY_SCANNED.clear()
+    NOTIFY_CUSTOM_MODULE_MAP.clear()
+
+    # Test a path that has no files to load in it
+    result = runner.invoke(cli.main, [
+        '--plugin-path', join(str(tmpdir), 'invalid_path'),
+        '-b', 'test\nbody',
+        'json://localhost',
+    ])
+    # The path is silently loaded but fails... it's okay because the
+    # notification we're choosing to notify does exist
+    assert result.exit_code == 0
+
+    # Directories that don't exist passed in by the CLI aren't even scanned
+    assert len(PATHS_PREVIOUSLY_SCANNED) == 0
+    assert len(NOTIFY_CUSTOM_MODULE_MAP) == 0
+
+    # Test our current existing path that has no entries in it
+    result = runner.invoke(cli.main, [
+        '--plugin-path', str(tmpdir.mkdir('empty')),
+        '-b', 'test\nbody',
+        'json://localhost',
+    ])
+    # The path is silently loaded but fails... it's okay because the
+    # notification we're choosing to notify does exist
+    assert result.exit_code == 0
+    assert len(PATHS_PREVIOUSLY_SCANNED) == 1
+    assert join(str(tmpdir), 'empty') in PATHS_PREVIOUSLY_SCANNED
+
+    # However there was nothing to load
+    assert len(NOTIFY_CUSTOM_MODULE_MAP) == 0
+
+    # Clear our working variables so they don't obstruct the next test
+    # This simulates an actual call from the CLI.  Unfortunately through
+    # testing were occupying the same memory space so our singleton's
+    # have already been populated
+    PATHS_PREVIOUSLY_SCANNED.clear()
+    NOTIFY_CUSTOM_MODULE_MAP.clear()
+
+    # Prepare ourselves a file to work with
+    notify_hook_a_base = tmpdir.mkdir('random')
+    notify_hook_a = notify_hook_a_base.join('myhook01.py')
+    notify_hook_a.write(cleandoc("""
+    raise ImportError
+    """))
+
+    result = runner.invoke(cli.main, [
+        '--plugin-path', str(notify_hook_a),
+        '-b', 'test\nbody',
+        # A custom hook:
+        'clihook://',
+    ])
+    # It doesn't exist so it will fail
+    # meanwhile we would have failed to load the myhook path
+    assert result.exit_code == 1
+
+    # The path is silently loaded but fails... it's okay because the
+    # notification we're choosing to notify does exist
+    assert len(PATHS_PREVIOUSLY_SCANNED) == 1
+    assert str(notify_hook_a) in PATHS_PREVIOUSLY_SCANNED
+    # However there was nothing to load
+    assert len(NOTIFY_CUSTOM_MODULE_MAP) == 0
+
+    # Prepare ourselves a file to work with
+    notify_hook_aa = notify_hook_a_base.join('myhook02.py')
+    notify_hook_aa.write(cleandoc("""
+    garbage entry
+    """))
+
+    result = runner.invoke(cli.main, [
+        '--plugin-path', str(notify_hook_aa),
+        '-b', 'test\nbody',
+        # A custom hook:
+        'clihook://',
+    ])
+    # It doesn't exist so it will fail
+    # meanwhile we would have failed to load the myhook path
+    assert result.exit_code == 1
+
+    # The path is silently loaded but fails...
+    # as a result the path stacks with the last
+    assert len(PATHS_PREVIOUSLY_SCANNED) == 2
+    assert str(notify_hook_a) in PATHS_PREVIOUSLY_SCANNED
+    assert str(notify_hook_aa) in PATHS_PREVIOUSLY_SCANNED
+    # However there was nothing to load
+    assert len(NOTIFY_CUSTOM_MODULE_MAP) == 0
+
+    # Clear our working variables so they don't obstruct the next test
+    # This simulates an actual call from the CLI.  Unfortunately through
+    # testing were occupying the same memory space so our singleton's
+    # have already been populated
+    PATHS_PREVIOUSLY_SCANNED.clear()
+    NOTIFY_CUSTOM_MODULE_MAP.clear()
+
+    # Prepare ourselves a file to work with
+    notify_hook_b = tmpdir.mkdir('goodmodule').join('__init__.py')
+    notify_hook_b.write(cleandoc("""
+    from apprise.decorators import notify
+
+    # We want to trigger on anyone who configures a call to clihook://
+    @notify(on="clihook")
+    def mywrapper(body, title, notify_type, *args, **kwargs):
+        # A simple test - print to screen
+        print("{}: {} - {}".format(notify_type, title, body))
+
+        # No return (so a return of None) get's translated to True
+    """))
+
+    result = runner.invoke(cli.main, [
+        '--plugin-path', str(tmpdir),
+        '-b', 'test body',
+        # A custom hook:
+        'clihook://',
+    ])
+
+    # We can detect the goodmodule (which has an __init__.py in it)
+    # so we'll load okay
+    assert result.exit_code == 0
+
+    # Let's see how things got loaded:
+    assert len(PATHS_PREVIOUSLY_SCANNED) == 2
+    assert str(tmpdir) in PATHS_PREVIOUSLY_SCANNED
+    # absolute path to detected module is also added
+    assert join(str(tmpdir), 'goodmodule', '__init__.py') \
+        in PATHS_PREVIOUSLY_SCANNED
+
+    # We also loaded our clihook properly
+    assert len(NOTIFY_CUSTOM_MODULE_MAP) == 1
+
+    # We can find our new hook loaded in our NOTIFY_SCHEMA_MAP now...
+    assert 'clihook' in NOTIFY_SCHEMA_MAP
+
+    # Store our key after parsing it as a list (this makes this test backwards
+    # compatible with Python 2.x
+    key = [k for k in NOTIFY_CUSTOM_MODULE_MAP.keys()][0]
+
+    assert len(NOTIFY_CUSTOM_MODULE_MAP[key]['notify']) == 1
+    assert 'clihook' in NOTIFY_CUSTOM_MODULE_MAP[key]['notify']
+
+    # Our function name
+    assert NOTIFY_CUSTOM_MODULE_MAP[key]['notify']['clihook']['fn_name'] \
+        == 'mywrapper'
+    # What we parsed from the `on` keyword in the @notify decorator
+    assert NOTIFY_CUSTOM_MODULE_MAP[key]['notify']['clihook']['url'] \
+        == 'clihook://'
+    # our default name Assignment.  This can be-overridden on the @notify
+    # decorator by just adding a name= to the parameter list
+    assert NOTIFY_CUSTOM_MODULE_MAP[key]['notify']['clihook']['name'] \
+        == 'Custom - clihook'
+
+    # Our Base Notification object when initialized:
+    assert isinstance(
+        NOTIFY_CUSTOM_MODULE_MAP[key]['notify']['clihook']['plugin'](),
+        NotifyBase)
+
+    # This is how it ties together in the backend
+    assert NOTIFY_CUSTOM_MODULE_MAP[key]['notify']['clihook']['plugin'] == \
+        NOTIFY_SCHEMA_MAP['clihook']
+
+    # Clear our working variables so they don't obstruct the next test
+    # This simulates an actual call from the CLI.  Unfortunately through
+    # testing were occupying the same memory space so our singleton's
+    # have already been populated
+    PATHS_PREVIOUSLY_SCANNED.clear()
+    NOTIFY_CUSTOM_MODULE_MAP.clear()
+    del NOTIFY_SCHEMA_MAP['clihook']
+
+    result = runner.invoke(cli.main, [
+        '--plugin-path', str(notify_hook_b),
+        '-b', 'test body',
+        # A custom hook:
+        'clihook://',
+    ])
+
+    # Absolute path to __init__.py is okay
+    assert result.exit_code == 0
+
+    # we can verify that it prepares our message
+    assert result.stdout.strip() == 'info:  - test body'
+
+    # Clear our working variables so they don't obstruct the next test
+    # This simulates an actual call from the CLI.  Unfortunately through
+    # testing were occupying the same memory space so our singleton's
+    # have already been populated
+    PATHS_PREVIOUSLY_SCANNED.clear()
+    NOTIFY_CUSTOM_MODULE_MAP.clear()
+    del NOTIFY_SCHEMA_MAP['clihook']
+
+    result = runner.invoke(cli.main, [
+        '--plugin-path', dirname(str(notify_hook_b)),
+        '-b', 'test body',
+        # A custom hook:
+        'clihook://',
+    ])
+
+    # Now we succeed to load our module when pointed to it only because
+    # an __init__.py is found on the inside of it
+    assert result.exit_code == 0
+
+    # we can verify that it prepares our message
+    assert result.stdout.strip() == 'info:  - test body'
+
+    # Test double paths that are the same; this ensures we only
+    # load the plugin once
+    result = runner.invoke(cli.main, [
+        '--plugin-path', dirname(str(notify_hook_b)),
+        '--plugin-path', str(notify_hook_b),
+        '--details',
+    ])
+
+    # Now we succeed to load our module when pointed to it only because
+    # an __init__.py is found on the inside of it
+    assert result.exit_code == 0
+
+    # Clear our working variables so they don't obstruct the next test
+    # This simulates an actual call from the CLI.  Unfortunately through
+    # testing were occupying the same memory space so our singleton's
+    # have already been populated
+    PATHS_PREVIOUSLY_SCANNED.clear()
+    NOTIFY_CUSTOM_MODULE_MAP.clear()
+    del NOTIFY_SCHEMA_MAP['clihook']
+
+    # Prepare ourselves a file to work with
+    notify_hook_b = tmpdir.mkdir('complex').join('complex.py')
+    notify_hook_b.write(cleandoc("""
+    from apprise.decorators import notify
+
+    # We can't over-ride an element that already exists
+    # in this case json://
+    @notify(on="json")
+    def mywrapper_01(body, title, notify_type, *args, **kwargs):
+        # Return True (same as None)
+        return True
+
+    @notify(on="willfail", name="always failing...")
+    def mywrapper_02(body, title, notify_type, *args, **kwargs):
+        # Simply fail
+        return False
+
+    @notify(on="clihook1", name="the original clihook entry")
+    def mywrapper_03(body, title, notify_type, *args, **kwargs):
+        # Return True
+        return True
+
+    # This is a duplicate o the entry above, so it can not be
+    # loaded...
+    @notify(on="clihook1", name="a duplicate of the clihook entry")
+    def mywrapper_04(body, title, notify_type, *args, **kwargs):
+        # Return True
+        return True
+
+    # This is where things get realy cool... we can not only
+    # define the schema we want to over-ride, but we can define
+    # some default values to pass into our wrapper function to
+    # act as a base before whatever was actually passed in is
+    # applied ontop.... think of it like templating information
+    @notify(on="clihook2://localhost")
+    def mywrapper_05(body, title, notify_type, *args, **kwargs):
+        # Return True
+        return True
+
+
+    # This can't load because of the defined schema/on definition
+    @notify(on="", name="an invalid schema was specified")
+    def mywrapper_06(body, title, notify_type, *args, **kwargs):
+        return True
+    """))
+
+    result = runner.invoke(cli.main, [
+        '--plugin-path', join(str(tmpdir), 'complex'),
+        '-b', 'test body',
+        # A custom hook that does not exist
+        'clihook://',
+    ])
+
+    # Since clihook:// isn't in our complex listing, this will fail
+    assert result.exit_code == 1
+
+    # Let's see how things got loaded
+    assert len(PATHS_PREVIOUSLY_SCANNED) == 2
+    # Our path we specified on the CLI...
+    assert join(str(tmpdir), 'complex') in PATHS_PREVIOUSLY_SCANNED
+
+    # absolute path to detected module is also added
+    assert join(str(tmpdir), 'complex', 'complex.py') \
+        in PATHS_PREVIOUSLY_SCANNED
+
+    # We loaded our one module successfuly
+    assert len(NOTIFY_CUSTOM_MODULE_MAP) == 1
+
+    # We can find our new hook loaded in our SCHEMA_MAP now...
+    assert 'willfail' in NOTIFY_SCHEMA_MAP
+    assert 'clihook1' in NOTIFY_SCHEMA_MAP
+    assert 'clihook2' in NOTIFY_SCHEMA_MAP
+
+    # Store our key after parsing it as a list (this makes this test backwards
+    # compatible with Python 2.x
+    key = [k for k in NOTIFY_CUSTOM_MODULE_MAP.keys()][0]
+
+    assert len(NOTIFY_CUSTOM_MODULE_MAP[key]['notify']) == 3
+    assert 'willfail' in NOTIFY_CUSTOM_MODULE_MAP[key]['notify']
+    assert 'clihook1' in NOTIFY_CUSTOM_MODULE_MAP[key]['notify']
+    # We only load 1 instance of the clihook2, the second will fail
+    assert 'clihook2' in NOTIFY_CUSTOM_MODULE_MAP[key]['notify']
+    # We can never load previously created notifications
+    assert 'json' not in NOTIFY_CUSTOM_MODULE_MAP[key]['notify']
+
+    result = runner.invoke(cli.main, [
+        '--plugin-path', join(str(tmpdir), 'complex'),
+        '-b', 'test body',
+        # A custom notification set up for failure
+        'willfail://',
+    ])
+    # Note that the failure of the decorator carries all the way back
+    # to the CLI
+    assert result.exit_code == 1
+
+    result = runner.invoke(cli.main, [
+        '--plugin-path', join(str(tmpdir), 'complex'),
+        '-b', 'test body',
+        # our clihook that returns true
+        'clihook1://',
+        # our other loaded clihook
+        'clihook2://',
+    ])
+    # Note that the failure of the decorator carries all the way back
+    # to the CLI
+    assert result.exit_code == 0
+
+    result = runner.invoke(cli.main, [
+        '--plugin-path', join(str(tmpdir), 'complex'),
+        # Print our custom details to the screen
+        '--details',
+    ])
+    assert 'willfail' in result.stdout
+    assert 'always failing...' in result.stdout
+
+    assert 'clihook1' in result.stdout
+    assert 'the original clihook entry' in result.stdout
+    assert 'a duplicate of the clihook entry' not in result.stdout
+
+    assert 'clihook2' in result.stdout
+    assert 'Custom - clihook2' in result.stdout
+
+    # Note that the failure of the decorator carries all the way back
+    # to the CLI
+    assert result.exit_code == 0
+
+
 @mock.patch('platform.system')
 def test_apprise_cli_windows_env(mock_system):
     """
