@@ -119,6 +119,20 @@ apprise_url_tests = (
         # user and token specified; image set to True
         'instance': plugins.NotifyMatrix,
     }),
+    # A Bunch of bad ports
+    ('matrixs://user:pass@hostname:port/#room_alias', {
+        # Invalid Port specified (was a string)
+        'instance': TypeError,
+    }),
+    ('matrixs://user:pass@hostname:0/#room_alias', {
+        # Invalid Port specified (was a string)
+        'instance': TypeError,
+    }),
+    ('matrixs://user:pass@hostname:65536/#room_alias', {
+        # Invalid Port specified (was a string)
+        'instance': TypeError,
+    }),
+    # More general testing...
     ('matrixs://user@{}?mode=t2bot&format=markdown&image=True'
      .format('a' * 64), {
          # user and token specified; image set to True
@@ -206,25 +220,26 @@ def test_plugin_matrix_general(mock_post, mock_get):
     mock_post.return_value = request
 
     # Variation Initializations
-    obj = plugins.NotifyMatrix(targets='#abcd')
+    obj = plugins.NotifyMatrix(host='host', targets='#abcd')
     assert isinstance(obj, plugins.NotifyMatrix) is True
     assert isinstance(obj.url(), six.string_types) is True
     # Registration successful
     assert obj.send(body="test") is True
 
-    obj = plugins.NotifyMatrix(user='user', targets='#abcd')
+    obj = plugins.NotifyMatrix(host='host', user='user', targets='#abcd')
     assert isinstance(obj, plugins.NotifyMatrix) is True
     assert isinstance(obj.url(), six.string_types) is True
     # Registration successful
     assert obj.send(body="test") is True
 
-    obj = plugins.NotifyMatrix(password='passwd', targets='#abcd')
+    obj = plugins.NotifyMatrix(host='host', password='passwd', targets='#abcd')
     assert isinstance(obj, plugins.NotifyMatrix) is True
     assert isinstance(obj.url(), six.string_types) is True
     # A username gets automatically generated in these cases
     assert obj.send(body="test") is True
 
-    obj = plugins.NotifyMatrix(user='user', password='passwd', targets='#abcd')
+    obj = plugins.NotifyMatrix(
+        host='host', user='user', password='passwd', targets='#abcd')
     assert isinstance(obj.url(), six.string_types) is True
     assert isinstance(obj, plugins.NotifyMatrix) is True
     # Registration Successful
@@ -279,17 +294,18 @@ def test_plugin_matrix_general(mock_post, mock_get):
     # Fails because we couldn't register because of 404 errors
     assert obj.send(body="test") is False
 
-    obj = plugins.NotifyMatrix(user='test', targets='#abcd')
+    obj = plugins.NotifyMatrix(host='host', user='test', targets='#abcd')
     assert isinstance(obj, plugins.NotifyMatrix) is True
     # Fails because we still couldn't register
     assert obj.send(user='test', password='passwd', body="test") is False
 
-    obj = plugins.NotifyMatrix(user='test', password='passwd', targets='#abcd')
+    obj = plugins.NotifyMatrix(
+        host='host', user='test', password='passwd', targets='#abcd')
     assert isinstance(obj, plugins.NotifyMatrix) is True
     # Fails because we still couldn't register
     assert obj.send(body="test") is False
 
-    obj = plugins.NotifyMatrix(password='passwd', targets='#abcd')
+    obj = plugins.NotifyMatrix(host='host', password='passwd', targets='#abcd')
     # Fails because we still couldn't register
     assert isinstance(obj, plugins.NotifyMatrix) is True
     assert obj.send(body="test") is False
@@ -317,7 +333,7 @@ def test_plugin_matrix_general(mock_post, mock_get):
     request.content = dumps(response_obj)
     request.status_code = requests.codes.ok
 
-    obj = plugins.NotifyMatrix(targets=None)
+    obj = plugins.NotifyMatrix(host='host', targets=None)
     assert isinstance(obj, plugins.NotifyMatrix) is True
 
     # Force a empty joined list response
@@ -377,14 +393,15 @@ def test_plugin_matrix_fetch(mock_post, mock_get):
     mock_post.side_effect = fetch_failed
 
     obj = plugins.NotifyMatrix(
-        user='user', password='passwd', include_image=True)
+        host='host', user='user', password='passwd', include_image=True)
     assert isinstance(obj, plugins.NotifyMatrix) is True
     # We would hve failed to send our image notification
     assert obj.send(user='test', password='passwd', body="test") is False
 
     # Do the same query with no images to fetch
     asset = AppriseAsset(image_path_mask=False, image_url_mask=False)
-    obj = plugins.NotifyMatrix(user='user', password='passwd', asset=asset)
+    obj = plugins.NotifyMatrix(
+        host='host', user='user', password='passwd', asset=asset)
     assert isinstance(obj, plugins.NotifyMatrix) is True
     # We would hve failed to send our notification
     assert obj.send(user='test', password='passwd', body="test") is False
@@ -412,7 +429,7 @@ def test_plugin_matrix_fetch(mock_post, mock_get):
     mock_post.return_value = request
     mock_get.return_value = request
 
-    obj = plugins.NotifyMatrix(include_image=True)
+    obj = plugins.NotifyMatrix(host='host', include_image=True)
     assert isinstance(obj, plugins.NotifyMatrix) is True
     assert obj.access_token is None
     assert obj._register() is True
@@ -465,7 +482,7 @@ def test_plugin_matrix_auth(mock_post, mock_get):
     mock_post.return_value = request
     mock_get.return_value = request
 
-    obj = plugins.NotifyMatrix()
+    obj = plugins.NotifyMatrix(host='localhost')
     assert isinstance(obj, plugins.NotifyMatrix) is True
     assert obj.access_token is None
     # logging out without an access_token is silently a success
@@ -502,7 +519,7 @@ def test_plugin_matrix_auth(mock_post, mock_get):
     assert obj.access_token is None
 
     # So will login
-    obj = plugins.NotifyMatrix(user='user', password='password')
+    obj = plugins.NotifyMatrix(host='host', user='user', password='password')
     assert isinstance(obj, plugins.NotifyMatrix) is True
     assert obj._login() is False
     assert obj.access_token is None
@@ -567,7 +584,7 @@ def test_plugin_matrix_rooms(mock_post, mock_get):
     mock_post.return_value = request
     mock_get.return_value = request
 
-    obj = plugins.NotifyMatrix()
+    obj = plugins.NotifyMatrix(host='host')
     assert isinstance(obj, plugins.NotifyMatrix) is True
     assert obj.access_token is None
 
@@ -626,7 +643,7 @@ def test_plugin_matrix_rooms(mock_post, mock_get):
 
     # Room creation
     request.status_code = requests.codes.ok
-    obj = plugins.NotifyMatrix()
+    obj = plugins.NotifyMatrix(host='host')
     assert isinstance(obj, plugins.NotifyMatrix) is True
     assert obj.access_token is None
 
@@ -671,7 +688,7 @@ def test_plugin_matrix_rooms(mock_post, mock_get):
     # Room detection
     request.status_code = requests.codes.ok
     request.content = dumps(response_obj)
-    obj = plugins.NotifyMatrix()
+    obj = plugins.NotifyMatrix(host='localhost')
     assert isinstance(obj, plugins.NotifyMatrix) is True
     assert obj.access_token is None
 
@@ -697,7 +714,7 @@ def test_plugin_matrix_rooms(mock_post, mock_get):
 
     # Room id lookup
     request.status_code = requests.codes.ok
-    obj = plugins.NotifyMatrix()
+    obj = plugins.NotifyMatrix(host='localhost')
     assert isinstance(obj, plugins.NotifyMatrix) is True
     assert obj.access_token is None
 
@@ -780,7 +797,7 @@ def test_plugin_matrix_image_errors(mock_post, mock_get):
     mock_get.side_effect = mock_function_handing
     mock_post.side_effect = mock_function_handing
 
-    obj = plugins.NotifyMatrix(include_image=True)
+    obj = plugins.NotifyMatrix(host='host', include_image=True)
     assert isinstance(obj, plugins.NotifyMatrix) is True
     assert obj.access_token is None
 
@@ -788,7 +805,7 @@ def test_plugin_matrix_image_errors(mock_post, mock_get):
     # we had post errors (of any kind) we still report a failure.
     assert obj.notify('test', 'test') is False
 
-    obj = plugins.NotifyMatrix(include_image=False)
+    obj = plugins.NotifyMatrix(host='host', include_image=False)
     assert isinstance(obj, plugins.NotifyMatrix) is True
     assert obj.access_token is None
 
@@ -817,13 +834,13 @@ def test_plugin_matrix_image_errors(mock_post, mock_get):
     # Prepare Mock
     mock_get.side_effect = mock_function_handing
     mock_post.side_effect = mock_function_handing
-    obj = plugins.NotifyMatrix(include_image=True)
+    obj = plugins.NotifyMatrix(host='host', include_image=True)
     assert isinstance(obj, plugins.NotifyMatrix) is True
     assert obj.access_token is None
 
     assert obj.notify('test', 'test') is True
 
-    obj = plugins.NotifyMatrix(include_image=False)
+    obj = plugins.NotifyMatrix(host='host', include_image=False)
     assert isinstance(obj, plugins.NotifyMatrix) is True
     assert obj.access_token is None
 
